

const string fwExternalApps_BASE_NODE = "ExternalApps";

const string fwExternalApps_TREEVIEWCONFIG_FILE = "fwExternalAppsTreeViewConfig.xml";

const string fwExternalApps_EXPERIMENT_VARIABLE = "{varExperiment}";
const string fwExternalApps_EXPERIMENT_VARIABLE_UPPERCASE = "{varExperimentUppercase}";


fwExternalApps_readXmlFile(string xmlPath, string &xmlFileContents, dyn_string &exceptionInfo)
{
  if(isfile(xmlPath))
    fileToString(xmlPath, xmlFileContents);
  else
    fwException_raise(exceptionInfo, "ERROR", "The XML configuration file could not be found (" + xmlPath + ")", "");      
}


// Daniel Davids - IT/CO - July 2008
// 
// This is the second part of the code which has been added to deal with the different way of handling
// the External-Application's XML file - They are now displayed and are editable before loading them

//      -1-      -2-   -3-   -4-   -5-   -6-   -7-   -8-   -9-   -10-  (....... Element-Data .......)
//
//                A            <---+       +--->      n <= m!
//                |                |       |
// +-----------+--|--+-----+-----+-|---+---|-+-----+-----+-----+-----+-----+--  ....  --+-----+-----+
// | "Element" | par | fst | lst | prv | nxt | act | who | nbr | dat |     |            |     |     |
// +-----------+-----+-/---+---\-+-----+-----+--|--+--|--+--|--+--|--+-----+--  ....  --+-----+-----+
//                    /         \               |     |     |     |
//                   V           V             0-1   <n>   <m>   0-1


                                                 //  Indexes in the Table starting at One
                                                 //  ------------------------------------
const int fwExternalApps_TREE_ELEMT_INDEX =  1;  //  The Tree Node Element String Itself
const int fwExternalApps_PARENTNODE_INDEX =  2;  //  Parent Node - Index in Table
const int fwExternalApps_FRST_CHILD_INDEX =  3;  //  First Child - Index in Table
const int fwExternalApps_LAST_CHILD_INDEX =  4;  //  Last Child - Index in Table
const int fwExternalApps_PREV_SIBLG_INDEX =  5;  //  Prev Sibling - Index in Table
const int fwExternalApps_NEXT_SIBLG_INDEX =  6;  //  Next Sibling - Index in Table
const int fwExternalApps_ELEM_ACTIV_INDEX =  7;  //  Tree Node Element Itself is Active?
const int fwExternalApps_CHILDN_ACT_INDEX =  8;  //  Nb of Children Currently Active
const int fwExternalApps_NBCHILDREN_INDEX =  9;  //  Number of Children which do Exist
const int fwExternalApps_NODESUBPUB_INDEX = 10;  //  Does the Tree Node Got Data (Subs/Publ)?

const int fwExternalApps_NUMB_OF_CONTROLS = 10;  // Total Number of 'dyn_int' Control Nodes

// These are Currently the Four Types for the 'fwExternalApps_NODESUBPUB_INDEX' Index
const int fwExternalApps_DATA_TYPE_NODATA =  0;  // No Data Associated
const int fwExternalApps_DATA_TYPE_SUBSCR =  1;  // Only carries Subscriptions
const int fwExternalApps_DATA_TYPE_PUBLIC =  2;  // Only carries Publications
const int fwExternalApps_DATA_TYPE_SUBPUB =  3;  // Subscriptions and Publications

// This extra Entry Beyond the Control is Used to Store the Associated stored Data
// This Presence of the Data is Defined by the 'fwExternalApps_NODESUBPUB_INDEX' Field
const int fwExternalApps_NODES_DATA_INDEX = 11;

// The Following Flag Defines the Number of Displayed Columns in the Tree Widget
const int fwExternalApps_DISPLAY_EXPANDED =  0;

// These are the Columns in the Tree Widget
const int fwExternalApps_COLUMN_MAIN = 0;
const int fwExternalApps_COLUMN__ACT = 1;
const int fwExternalApps_COLUMN_SUBS = 2;
const int fwExternalApps_COLUMN__ALL = 3;
const int fwExternalApps_COLUMN_SOME = 4;
const int fwExternalApps_COLUMN_NONE = 5;

// These are the Icons Used in the Tree Widget that Denote the (Des)Active State
const string fwExternalApps_COLUMN_ASSERT_ICON = "checked.png";
const string fwExternalApps_COLUMN_DELETE_ICON = "delete.png";

// The Following is a Debug-Flag which Checks Performances
const int fwExternalApps_CHECK_PERFORMANCE  =  0;


// The Indexes into the Application Data's Dynamic Array

const int fwExternalApps_DATA_SUBS_PUBL     =  1;
const int fwExternalApps_DATA_SUBPUB_NAME   =  2;
const int fwExternalApps_DATA_SUBPUB_TYPE   =  3;
const int fwExternalApps_DATA_BUFF_MODE     =  4;
const int fwExternalApps_DATA_OVER_WRITE    =  5;
const int fwExternalApps_DATA_PNT_NAME      =  6;
const int fwExternalApps_DATA_PNT_ALIAS     =  7;
const int fwExternalApps_DATA_PNT_DESCRIP   =  8;
const int fwExternalApps_DATA_PNT_ELEMENT   =  9;
const int fwExternalApps_DATA_PNT_TAGNAME   = 10;
const int fwExternalApps_DATA_PNT_TYPNAME   = 11;
const int fwExternalApps_DATA_PNT_UNITS     = 12;
const int fwExternalApps_DATA_PNT_MINERROR  = 13;
const int fwExternalApps_DATA_PNT_MINWARN   = 14;
const int fwExternalApps_DATA_PNT_MAXWARN   = 15;
const int fwExternalApps_DATA_PNT_MAXERROR  = 16;
    

const string fwExternalApps_DIPCONFIG_TYPE = "_FwDipConfig";
const string fwExternalApps_DIPCONFIG_NAME = "DIPConfig_1";
const int    fwExternalApps_D_QUERYTIMEOUT = 15;


/*
    The following functions are purely used for Debugging purposes
    
    _fwExternalApps_printTable prints out all the Control information of the Table
    
*/


int _fwExternalApps_printTable(dyn_dyn_anytype & tbl)
{
int siz = dynlen(tbl);
string lines;
int child;

  DebugTN("Table Dump");
  for ( child = 1 ; child <= siz ; ++child )
  {
    sprintf(lines,"Entry %2d - %-16s | par %2d - fst %2d - lst %2d - prv %2d - nxt %2d",
            child,tbl[child][fwExternalApps_TREE_ELEMT_INDEX],tbl[child][fwExternalApps_PARENTNODE_INDEX],
                  tbl[child][fwExternalApps_FRST_CHILD_INDEX],tbl[child][fwExternalApps_LAST_CHILD_INDEX],
                  tbl[child][fwExternalApps_PREV_SIBLG_INDEX],tbl[child][fwExternalApps_NEXT_SIBLG_INDEX]);
    sprintf(lines,"%s | active %s - nbr %2d - who %2d - datas %s",
            lines,(tbl[child][fwExternalApps_ELEM_ACTIV_INDEX]==0?" No":"Yes"),
                   tbl[child][fwExternalApps_CHILDN_ACT_INDEX],
                   tbl[child][fwExternalApps_NBCHILDREN_INDEX],
                  (tbl[child][fwExternalApps_NODESUBPUB_INDEX]==fwExternalApps_DATA_TYPE_NODATA?"   -":
                  (tbl[child][fwExternalApps_NODESUBPUB_INDEX]==fwExternalApps_DATA_TYPE_SUBSCR?"Subs":
                  (tbl[child][fwExternalApps_NODESUBPUB_INDEX]==fwExternalApps_DATA_TYPE_PUBLIC?"Publ":
                  (tbl[child][fwExternalApps_NODESUBPUB_INDEX]==fwExternalApps_DATA_TYPE_SUBPUB?" S/P":"Bug!")))));
    DebugN(lines);
  }
}
  

/*
    The following routines initialise a Table that controls a Tree Widget's structure
    fwExternalApps_initialiseTable initialises a table
    _fwExternalApps_createNewTableEntry Creates a new entry in the Table
    fwExternalApps_readsDevicesFromString reads all the Subscriptions/Publications from a string (was a file)
    fwExternalApps_createDevicesFromTree creates all devices from information hold in the Tree widget's structure
    fwExternalApps_insertElementInTable & _fwExternalApps_insertDataElement inserts a canonical element in the Table
    _fwExternalApps_mergeDataInTable merges the application data in the Table
    fwExternalApps_showNodeDescription displays the Node's description
*/


// New Function that implements the reading of the Devices into the Database and Tree widget to be viewed

int fwExternalApps_initialiseTable(dyn_dyn_anytype & tbl, dyn_string & exceptionInfo)
{
int root;
  
  tbl = makeDynAnytype();
  root = _fwExternalApps_createNewTableEntry(tbl, "");
  
  tbl[root][fwExternalApps_ELEM_ACTIV_INDEX] = 1;
  
  // This Flag is Set so that One Can Insert New Entries in the Table
  // This Flag will be Cleared Once the Tree Widget Gets Populated
  tbl[1][fwExternalApps_NODES_DATA_INDEX] = 1;

  return ( root );
}


int _fwExternalApps_createNewTableEntry ( dyn_dyn_anytype & tbl , string element )
{
int siz;
dyn_anytype tblentry;

  tblentry = makeDynAnytype(element);
  for ( int i = 2 ; i <= fwExternalApps_NUMB_OF_CONTROLS ; ++i ) dynAppend ( tblentry , 0 );
  
  siz = dynlen(tbl);
  ++siz;
  tbl[siz] = tblentry;
    
  return ( siz );
}
    

void _fwExternalApps_readStructDataDefinitions ( int doc , int topchild , dyn_string & listTypeStructNames ,  dyn_string & listTypeStructTypes )
{
int idx, cnt, node, rtn_code;
dyn_int dipTypeNodes, allnodes;
string dipstructname, dipname, diptype;
dyn_mapping attrList;
dyn_string nameList, valsList;
dyn_string exceptionInfo;

  xmlChildNodes ( doc , topchild , dipTypeNodes );
  
  for ( idx = 1 ; idx <= dynlen(dipTypeNodes) ; ++idx )
  {
    node = dipTypeNodes[idx];
    
    if ( xmlNodeName ( doc , node ) != "dipType" ) continue;
    
    if ( xmlGetElementAttribute ( doc , node , "type" , dipstructname ) == 0 )
    {
      // DebugTN("dipstructname = '"+dipstructname+"'");
      
      xmlChildNodes ( doc , node , allnodes );
      
      if ( ( dynlen(allnodes) != 1 ) || ( xmlNodeName ( doc , allnodes[1] ) != "xsd:complexType" ) ) continue;
      
      node = allnodes[1];
      
      xmlChildNodes ( doc , node , allnodes );
      
      if ( ( dynlen(allnodes) != 1 ) || ( xmlNodeName ( doc , allnodes[1] ) != "xsd:all" ) ) continue;
      
      node = allnodes[1];
      
      xmlChildNodes ( doc , node , allnodes );
      
      rtn_code = fwXml_childNodesContent ( doc , node , nameList , attrList , valsList , exceptionInfo );
      
      if ( rtn_code != 0 ) continue;
      
      for ( cnt = 1 ; cnt <= dynlen(nameList) ; ++cnt )
      {
        if ( nameList[cnt] != "xsd:element" ) continue;
        if ( ! mappingHasKey ( attrList[cnt] , "name" ) ) continue;
        if ( ! mappingHasKey ( attrList[cnt] , "type" ) ) continue;
       
        dipname = attrList[cnt]["name"];
        diptype = attrList[cnt]["type"];
        
        if ( substr(diptype,0,4) == "xsd:" ) diptype = substr(diptype,4);
        
        // DebugTN("   Dip Name '"+dipname+"' - Dip Type '"+diptype+"'");
        
        dynAppend ( listTypeStructNames , dipstructname + "_" + dipname );
        dynAppend ( listTypeStructTypes , diptype );
      }
    }
  }  
}


void fwExternalApps_readsDevicesFromXmlStr ( string xmlFileContents , 
                    string & configName , string & publishName , int & queryTimeout ,
                    bool popAsDPs , bool popAsApp , dyn_dyn_anytype & xmltable , dyn_string & exceptionInfo )
{
string errMsg;
int errLin, errCol;
int document, topchild, rtn_code;
dyn_mapping attrList, spAttrList, dpeAttrList;
dyn_string nameList, spNameList, dpeNameList;
dyn_string valsList, spValsList, dpeValsList;
dyn_string listTypeStructNames, listTypeStructTypes;
string sSubscrPublicCommand, sSubscrPublicName, sSubscrPublicType;
dyn_string treeElems, applicationData;
string sBufferMode, sOverwriteMode;
int sSubscrPublic;
int dipTypeDepth, dipPosition;
string dipTypeValue;
          
  listTypeStructNames = makeDynString();
  listTypeStructTypes = makeDynString();
  
  if ( fwExternalApps_CHECK_PERFORMANCE ) DebugTN("Starts fwExternalApps_readsDevicesFromXmlStr Parsing");

  document = xmlDocumentFromString ( xmlFileContents , errMsg , errLin , errCol );
  // DebugTN ( "document = " + document );

  if ( fwExternalApps_CHECK_PERFORMANCE ) DebugTN("Ending fwExternalApps_readsDevicesFromXmlStr Parsing");

  if ( document < 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "Xml Parse Error '"+errMsg+"' Line="+errLin+" Column="+errCol, "");
    return;
  }
  
  if ( fwExternalApps_CHECK_PERFORMANCE ) DebugTN("Starts fwExternalApps_readsDevicesFromXmlStr ConfNam");

  topchild = xmlFirstChild ( document );
  // DebugTN ( "topchild = " + topchild );
  
  while (  ( topchild != -1 )
        && ( xmlNodeType ( document , topchild ) != XML_ELEMENT_NODE ) )
  {
    topchild = xmlNextSibling ( document , topchild );
    // DebugTN ( "topchild = " + topchild );
  }

  if ( topchild != -1 )
  {
    if ( xmlNodeName ( document , topchild ) == "contract" )
    {
      topchild = xmlFirstChild ( document , topchild );
    
      while (  ( topchild != -1 )
            && (  ( xmlNodeType ( document , topchild ) != XML_ELEMENT_NODE ) 
               || ( xmlNodeName ( document , topchild ) != "configuration" ) ) )
      {
        if ( ! popAsApp && ( xmlNodeName ( document , topchild ) == "structuredDataDefinitions" ) )
        {
          _fwExternalApps_readStructDataDefinitions ( document , topchild , listTypeStructNames , listTypeStructTypes );
        }

        topchild = xmlNextSibling ( document , topchild );
        // DebugTN ( "topchild = " + topchild );
      }
    }
  }
    
  if (  ( topchild == -1 )
     || ( xmlNodeName ( document , topchild ) != "configuration" ) )
  {
    fwException_raise(exceptionInfo, "ERROR", "Xml File's Top-Node must be 'contract' or 'configuration'", "");
    topchild = -1;
  }
  else
  {
    string tmp_configName = "";
    if ( xmlGetElementAttribute ( document , topchild , "name" , tmp_configName ) != -1 )
    {
      if ( tmp_configName == "" )
      {
        fwException_raise(exceptionInfo, "ERROR", "Xml File's 'configuration' 'name' attribute is empty", "");
        topchild = -1;
      }
      configName = tmp_configName;
    }
    else configName = fwExternalApps_DIPCONFIG_NAME;
    
    string tmp_publishName = "";
    if ( xmlGetElementAttribute ( document , topchild , "publisherName" , tmp_publishName ) != -1 )
    {
      publishName = tmp_publishName;
    }
    else publishName = "";
    
    int tmp_queryTimeout;
    string tmp_queryTimeout_str = 0;
    if ( xmlGetElementAttribute ( document , topchild , "queryTimeout" , tmp_queryTimeout_str ) != -1 )
    {
      tmp_queryTimeout = (int)tmp_queryTimeout_str;
      queryTimeout = ( tmp_queryTimeout > 0 ? tmp_queryTimeout : fwExternalApps_D_QUERYTIMEOUT );
    }
    else queryTimeout = 0;
  }
  
  // DebugTN("Import-File's Configuration",configName,publishName,queryTimeout);
  
  if ( topchild == -1 )
  {
    rtn_code = xmlCloseDocument ( document );
    // DebugTN ( "rtn_code = " + rtn_code );
  }

  if ( fwExternalApps_CHECK_PERFORMANCE ) DebugTN("Ending fwExternalApps_readsDevicesFromXmlStr ConfNam");

  if ( topchild == -1 ) { return; }
  
  if ( fwExternalApps_CHECK_PERFORMANCE ) DebugTN("Starts fwExternalApps_readsDevicesFromXmlStr Analyse");
   
  rtn_code = fwXml_childNodesContent ( document , topchild ,
                   nameList , attrList , valsList , exceptionInfo );
  
  // DebugTN ( "rtn_code = " + rtn_code );
  
  if ( rtn_code == 258 )
  {
    for ( int i = dynlen(nameList) ; i > 0 ; --i )
    {
      if ( nameList[i] == "#comment" ) { dynRemove(nameList,i); dynRemove(attrList,i); dynRemove(valsList,i); }
    }
    
    rtn_code = 2;
  }
  
  if ( rtn_code != 2 )
  {
    fwException_raise(exceptionInfo, "ERROR", "Xml File's 'configuration' may Only contain Elements", "");
  }
  else
  {
    int listSize = dynlen ( nameList );
    for ( int index = 1 ; index <= listSize ; ++index )
    {
      if ( ! mappingHasKey(attrList[index],fwXml_CHILDSUBTREEID) )
      {
        fwException_raise(exceptionInfo, "ERROR", "Xml File's 'configuration' may Only contain Elements", "");
        break;
      }
      
      topchild = attrList[index][fwXml_CHILDSUBTREEID];
          
      sSubscrPublicCommand = nameList[index];
      
      if      ( sSubscrPublicCommand == "subscription" ) sSubscrPublic = fwExternalApps_DATA_TYPE_SUBSCR;
      else if ( sSubscrPublicCommand == "publication" ) sSubscrPublic = fwExternalApps_DATA_TYPE_PUBLIC;
      else
      {
        fwException_raise(exceptionInfo, "ERROR", "Xml File's 'configuration' may Only contain 'subscription' and 'publication'", "");
        break;
      }

      if ( mappingHasKey(attrList[index],"name") )
         { sSubscrPublicName = attrList[index]["name"]; }
      else { sSubscrPublicName = "default"; }
      
      if ( mappingHasKey(attrList[index],"type") )
         { sSubscrPublicType = attrList[index]["type"]; }
      else { sSubscrPublicType = "multiple"; }

      if ( ! popAsApp )
      {
        if ( mappingHasKey(attrList[index],"xsdType") )
        {
          dipTypeDepth = 0;
          dipTypeValue = attrList[index]["xsdType"];
          
          if ( substr(dipTypeValue,0,4) == "xsd:" ) dipTypeValue = substr(dipTypeValue,4);
        }
        else if ( mappingHasKey(attrList[index],"dipType") )
        {
          dipTypeDepth = 1;
          dipTypeValue = attrList[index]["dipType"];
        }
        else
        {
          dipTypeDepth = -1;
          dipTypeValue = "";
        }
      }

      rtn_code = fwXml_childNodesContent ( document , topchild ,
                       spNameList , spAttrList , spValsList , exceptionInfo );

      sBufferMode = "0";
      sOverwriteMode = "FALSE";

      int spSize = dynlen ( spNameList );
      for ( int sp = 1 ; sp <= spSize ; ++sp )
      {
        if ( spNameList[sp] == "buffer" )
        {
          sBufferMode = spValsList[sp];
          
          dynRemove(spNameList,sp);
          dynRemove(spAttrList,sp);
          dynRemove(spValsList,sp);
          --sp; --spSize;
        }
        else if ( spNameList[sp] == "overwrite" )
        {
          sOverwriteMode = spValsList[sp];
          
          dynRemove(spNameList,sp);
          dynRemove(spAttrList,sp);
          dynRemove(spValsList,sp);
          --sp; --spSize;
        }
        else if ( spNameList[sp] == "#comment" )
        {
          dynRemove(spNameList,sp);
          dynRemove(spAttrList,sp);
          dynRemove(spValsList,sp);
          --sp; --spSize;
        }
      }
      
      /* * /
      DebugN((sSubscrPublic==fwExternalApps_DATA_TYPE_SUBSCR?"Subscription":"Publication"));
      DebugN("Name   "+sSubscrPublicName);
      DebugN("Type   "+sSubscrPublicType);
      DebugN("Buffer "+sBufferMode);
      DebugN("O/W    "+(sOverwriteMode?"True":"False"));
      / * */
      
      for ( int sp = 1 ; sp <= spSize ; ++sp )
      {
        if ( spNameList[sp] != "dpe" )
        {
          fwException_raise(exceptionInfo, "WARNING", "Xml File's 'subscr/public' contains unknown '"+spNameList[sp]+"' object", "");
          continue;
        }
        
        if ( ! mappingHasKey(spAttrList[sp],fwXml_CHILDSUBTREEID) )
        {
          fwException_raise(exceptionInfo, "WARNING", "Xml File's 'sunscr/public' may Only contain Elements", "");
          continue;
        }

        topchild = spAttrList[sp][fwXml_CHILDSUBTREEID];

        rtn_code = fwXml_childNodesContent ( document , topchild ,
                   dpeNameList , dpeAttrList , dpeValsList , exceptionInfo );

        if ( rtn_code != 0 )
        {
          fwException_raise(exceptionInfo, "WARNING", "Xml File's 'subscr/public' contains badly-formed 'dpe' declaration", "");
          continue;
        }

        mapping tempMapp;
        
        mappingClear ( tempMapp );
        
        tempMapp["name"]        = "";
        tempMapp["alias"]       = "";
        tempMapp["description"] = "";
        tempMapp["element"]     = ".";
        tempMapp["tag"]         = "default";
        tempMapp["type"]        = "";
        tempMapp["unit"]        = "";
        tempMapp["minError"]    = "";
        tempMapp["minWarn"]     = "";
        tempMapp["maxWarn"]     = "";
        tempMapp["maxError"]    = "";
        
        int dpeSize = dynlen ( dpeNameList );
        for ( int dpe = 1 ; dpe <= dpeSize ; ++dpe )
        {
          tempMapp[dpeNameList[dpe]] = dpeValsList[dpe];
        }
        
        /* * /
        string sp_idx;
        sprintf(sp_idx,"%-7d",sp);
        
        DebugN(sp_idx+"Name:     "+tempMapp["name"]);
        DebugN("       Alias:    "+tempMapp["alias"]);
        DebugN("       Descr:    "+tempMapp["description"]);
        DebugN("       Element:  "+tempMapp["element"]);
        DebugN("       Tag:      "+tempMapp["tag"]);
        DebugN("       Type:     "+tempMapp["type"]);
        DebugN("       Unit:     "+tempMapp["unit"]);
        
        DebugN("       MinError: "+tempMapp["minError"]);
        DebugN("       MinWarn:  "+tempMapp["minWarn"]);
        DebugN("       MaxWarn:  "+tempMapp["maxWarn"]);
        DebugN("       MaxError: "+tempMapp["maxError"]);
        / * */

        /*
           Note that the following logically needs to be done for each Datapoint-Element as
           first of all the Datapoint-Name needs Not to be unique for the Same Subsc/Public,
           and secondly, the Datapoint-Alias might change the outcome of the checks below!
           
           if DP-Name not empty
              if DP-Alias empty
                 do nothing
              elsif DP-Alias does Not map to an existing DP-Name
                 do nothing
              elsif DP-Alias maps to the same DP-Name
                 do nothing
              else {DP-Alias maps to a different DP-Name}
                 ignore this entry and raise an exception
           else {DP-Name empty}
              if DP-Alias empty
                 ignore this entry and raise an exception
              elsif DP-Alias does Not map to an existing DP-Name
                 ignore this entry and raise an exception
              else {DP-Alias maps to a DP-Name}
                 Use the translated DP-Name
        */
               
        if ( popAsApp )
        {
          dyn_string result;
          
          if ( tempMapp["name"] != "" )
          {
            if ( tempMapp["alias"] != "" )
            {
              result = dpNames("@"+tempMapp["alias"]);
              if ( ( dynlen(result) == 1 ) && ( tempMapp["name"] != result[1] ) )
              {
                fwException_raise(exceptionInfo, "WARNING", "DP-Name '"+tempMapp["name"]+"' and DP-Alias '"+tempMapp["alias"]+"' mismatched to '"+result[1]+"'", "");
                continue;
              }
            }
          }
          else
          {
            if ( tempMapp["alias"] == "" )
            {
              fwException_raise(exceptionInfo, "WARNING", "Empty DP-Name and DP-Alias for Subscr/Public '"+sSubscrPublicName+"'", "");
              continue;
            } 
          
            result = dpNames("@"+tempMapp["alias"]);
            if ( dynlen(result) != 1 )
            {
              fwException_raise(exceptionInfo, "WARNING", "Empty DP-Name and DP-Alias '"+tempMapp["alias"]+"' does Not Map to a DP-Name", "");
              continue;
            }

            result[1] = substr(result[1],0,strlen(result[1])-1);
            // DebugTN("Alias to Name '"+result[1]+"'");
            tempMapp["name"] = result[1];
          }
        }
        else
        {
          tempMapp["name"] = fwExternalApps_BASE_NODE + "/" + sSubscrPublicName + "/" + tempMapp["tag"];
          strreplace(tempMapp["name"],".","_");
          strreplace(tempMapp["name"],":","_");
          strreplace(tempMapp["name"],"[","_");
          strreplace(tempMapp["name"],"]","_");
          strreplace(tempMapp["name"],"$","_");
          tempMapp["alias"] = "";
          tempMapp["element"] = ".value";
          
          if ( dipTypeDepth == 0 ) tempMapp["type"] = "FwExternalAppsDIP" + dipTypeValue;
          
          if ( dipTypeDepth > 0 )
          {
            dipPosition = dynContains ( listTypeStructNames , dipTypeValue + "_" + tempMapp["tag"] );
            
            if ( dipPosition > 0 )
            {
               tempMapp["type"] = "FwExternalAppsDIP" + listTypeStructTypes[dipPosition];
            }
            else
            {
              dipTypeDepth = -1;
              dipTypeValue = "";
            }
          }
          
          if ( dipTypeDepth < 0 )
          {
              fwException_raise(exceptionInfo, "WARNING", "Type of DIP-Subscr/Public could not be Resolved '"+tempMapp["name"]+"'", "");
              continue;
          }
        }

        applicationData = makeDynString ( sSubscrPublic , sSubscrPublicName , sSubscrPublicType , sBufferMode , sOverwriteMode ,
                                          tempMapp["name"] , tempMapp["alias"] , tempMapp["description"] , tempMapp["element"] , 
                                          tempMapp["tag"] , tempMapp["type"] , tempMapp["unit"] ,
                                          tempMapp["minError"] , tempMapp["minWarn"] , tempMapp["maxWarn"] , tempMapp["maxError"] );
        
        // This makes the difference between creating the tree as a DP-tree or a DIP-tree.
        if ( popAsDPs )
        {
           treeElems = strsplit ( tempMapp["name"] , fwDevice_HIERARCHY_SEPARATOR );
        }
        else
        {
          treeElems = strsplit ( sSubscrPublicName , fwDevice_HIERARCHY_SEPARATOR );
        }

        fwExternalApps_insertElementInTable ( xmltable , treeElems , sSubscrPublic , applicationData , exceptionInfo );
      }
    }
  }
  
  rtn_code = xmlCloseDocument ( document );
  // DebugTN ( "rtn_code = " + rtn_code );

  if ( fwExternalApps_CHECK_PERFORMANCE ) DebugTN("Ending fwExternalApps_readsDevicesFromXmlStr Analyse");
}


int fwExternalApps_readTreeViewConfigFile ( bool & popAsDPs , bool & popAsApp , 
                                            dyn_string & nameList , dyn_mapping & attrList , dyn_string & valsList , 
                                            dyn_string & exceptionInfo )
{
int cnt;
string flag;
string errMsg;
int errLin, errCol;
int document, topchild, rtn_code;
string xmlFileName;
  
  xmlFileName = getPath ( DATA_REL_PATH , "fwExternalApps/" + fwExternalApps_TREEVIEWCONFIG_FILE );
  
  if ( xmlFileName == "" )
  {
    fwException_raise(exceptionInfo, "ERROR", "Tree-View Config File '"+fwExternalApps_TREEVIEWCONFIG_FILE+"' could Not be Found!", "");
    return ( -1 );
  }
  
  document = xmlDocumentFromFile ( xmlFileName , errMsg , errLin , errCol );

  if ( document < 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "Xml Parse Error '"+errMsg+"' Line="+errLin+" Column="+errCol, "");
    return ( -1 );
  }
  
  topchild = xmlFirstChild ( document );
  
  while ( ( topchild != -1 ) && ( xmlNodeType ( document , topchild ) != XML_ELEMENT_NODE ) )
  {
    topchild = xmlNextSibling ( document , topchild );
  }

  if ( ( topchild == -1 ) || ( xmlNodeName ( document , topchild ) != "ExternalAppsTreeViewConfig" ) )
  {
    fwException_raise(exceptionInfo, "ERROR", "Tree-View Config File's Top-Node must be 'ExternalAppsTreeViewConfig'", "");
    xmlCloseDocument ( document );
    return ( -1 );
  }

  rtn_code = fwXml_childNodesContent ( document , topchild , nameList , attrList , valsList , exceptionInfo );
      
  if ( ( rtn_code != 0 ) && ( rtn_code != 256 ) )
  {
    fwException_raise(exceptionInfo, "ERROR", "Tree-View Config File contains unacceptable contents", "");
    xmlCloseDocument ( document );
    return ( -1 );
  }

  // DebugTN ( "rtn_code = " + rtn_code );
  // DebugTN ( nameList , attrList , valsList );
    
  for ( cnt = 1 ; cnt <= dynlen(nameList) ; ++cnt )
  {
    if ( nameList[cnt] == "Populate_As_DPs" )
    {
      flag = substr(valsList[cnt],0,1);
      
      if ( flag == "T" ) popAsDPs = TRUE;
      if ( flag == "F" ) popAsDPs = FALSE;
    }
    
    if ( nameList[cnt] == "Populate_As_App" )
    {
      flag = substr(valsList[cnt],0,1);
      
      if ( flag == "T" ) popAsApp = TRUE;
      if ( flag == "F" ) popAsApp = FALSE;
    }
  }
  
  rtn_code = xmlCloseDocument ( document );
  
  return ( 0 );
}


int fwExternalApps_writeTreeViewConfigFile ( bool popAsDPs , bool popAsApp , 
                                             dyn_string nameList , dyn_mapping attrList , dyn_string valsList , 
                                             dyn_string & exceptionInfo )
{
int cnt;
string flag;
string errMsg;
int errLin, errCol;
int document, topchild, rtn_code;
string xmlFileName;
  
  for ( cnt = 1 ; cnt <= dynlen(nameList) ; ++cnt )
  {
    if ( nameList[cnt] == "Populate_As_DPs" ) valsList[cnt] = ( popAsDPs ? "True" : "False" );
    
    if ( nameList[cnt] == "Populate_As_App" ) valsList[cnt] = ( popAsApp ? "True" : "False" );
  }
  
  xmlFileName = getPath ( DATA_REL_PATH , "fwExternalApps/" + fwExternalApps_TREEVIEWCONFIG_FILE );
  
  if ( xmlFileName == "" )
  {
    fwException_raise(exceptionInfo, "ERROR", "Tree-View Config File '"+fwExternalApps_TREEVIEWCONFIG_FILE+"' could Not be Found!", "");
    return ( -1 );
  }
  
  document = xmlNewDocument ( xmlFileName , errMsg , errLin , errCol );

  if ( document < 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "Could not create XML document in memory!", "");
    return ( -1 );
  }
  
  topchild = xmlAppendChild ( document , -1 , XML_ELEMENT_NODE , "ExternalAppsTreeViewConfig" );
  
  if ( topchild < 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "Could not add the topchild to the XML document!", "");
    xmlCloseDocument ( document );
    return ( -1 );
  }

  rtn_code = fwXml_appendChildContent ( document , topchild , nameList , attrList , valsList , exceptionInfo );
      
  if ( rtn_code != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "Could not add list of children to the XML document!", "");
    xmlCloseDocument ( document );
    return ( -1 );
  }
 
  rtn_code = xmlDocumentToFile ( document , xmlFileName );
  
  if ( rtn_code != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "Could not write XML document to the file '"+xmlFileName+"'", "");
    xmlCloseDocument ( document );
    return ( -1 );
  }
   
  rtn_code = xmlCloseDocument ( document );
  
  return ( 0 );
}


void fwExternalApps_createDevicesFromTree ( bool popAsDPs , dyn_dyn_anytype & tbl , 
                                            string configName , string publishName , int queryTimeout ,
                                            string systemType , dyn_string nodeTypes , dyn_string & exceptionInfo )
{
int tracing = 1;
int i, idx, index, cnt, ent, subpub_sort, subpub_buff, subpub_over;
string strException;
dyn_int dpsToCrDept;
dyn_string elems, dpsSubsProd, dpsToCreate, dpsToCrType, dpsToParent, dpeToCrLeaf;
dyn_dyn_string thedata;
string subpub_item, subpub_tagn, subpub_type, dpelem_name;
dyn_string sub_name_all_0 = makeDynString();
dyn_string sub_item_all_0 = makeDynString();
dyn_string sub_tags_all_0 = makeDynString();
dyn_string sub_name_all_1 = makeDynString();
dyn_string sub_item_all_1 = makeDynString();
dyn_string sub_tags_all_1 = makeDynString();
dyn_string pub_sgl_name_all_0 = makeDynString();
dyn_string pub_sgl_item_all_0 = makeDynString();
dyn_int    pub_sgl_buff_all_0 = makeDynInt();
dyn_string pub_sgl_name_all_1 = makeDynString();
dyn_string pub_sgl_item_all_1 = makeDynString();
dyn_int    pub_sgl_buff_all_1 = makeDynInt();
dyn_dyn_anytype pub_str_all = makeDynAnytype();
int pub_str_0 = 0;
int pub_str_1 = 0;
int pub_tag_0 = 0;
int pub_tag_1 = 0;

  fwDevice_initialize();

  if ( tracing > 0 )
  {
    if ( dynlen(nodeTypes) > 0 )
    {
      DebugTN("The following Levels correspond to the following Datapoint-Types");
      DebugTN("Datapoint Types: '"+(string)nodeTypes+"'");
    }
  
    DebugTN("DIP Configuration Type is '"+fwExternalApps_DIPCONFIG_TYPE+"'");
    DebugTN("DIP Configuration Name is '"+configName+"'");
  }
  
  if ( configName == "" )
  {
    fwException_raise(exceptionInfo, "WARNING", "DIP Configuration Name Not Known!", "");
    return;
  }

  if ( dpExists(configName) == FALSE )
  {
    DebugTN("Creating '"+configName+"' of type '"+fwExternalApps_DIPCONFIG_TYPE+"'");

    dpCreate ( configName , fwExternalApps_DIPCONFIG_TYPE );
    dyn_errClass errors = getLastError();
    if ( dynlen(errors) > 0 )
    {
      fwException_raise(exceptionInfo, "WARNING", "DIP Datapoint '"+configName+"' Could NOT be Created", "");
      return;
    }
    while(!dpExists(configName)) delay(0,100);
    
    if ( publishName == "" ) publishName = getHostname();
    publishName += "_" + rand();
    dpSet(configName+".publishName",publishName);
    
    if ( queryTimeout <= 0 ) queryTimeout = fwExternalApps_D_QUERYTIMEOUT;
    dpSet(configName+".queryTimeout",queryTimeout);
  }
  else
  {
    string tmp_publishName;
    dpGet(configName+".publishName",tmp_publishName);
    if ( tmp_publishName == "" )
    {
      if ( publishName == "" ) publishName = getHostname();
      publishName += "_" + rand();
      dpSet(configName+".publishName",publishName);
    }
    else
    {
      if ( publishName != "" )
      {
        if ( publishName != substr(tmp_publishName,0,strlen(publishName)) )
        {
          publishName += "_" + rand();
          dpSet(configName+".publishName",publishName);
        }
      }
    }
    
    int tmp_queryTimeout;
    dpGet(configName+".queryTimeout",tmp_queryTimeout);
    if ( tmp_queryTimeout <= 0 )
    {
      if ( queryTimeout <= 0 ) queryTimeout = fwExternalApps_D_QUERYTIMEOUT;
      dpSet(configName+".queryTimeout",queryTimeout);
    }
    else
    {
      if ( queryTimeout > 0 )
      {
        if ( queryTimeout != tmp_queryTimeout )
        {
          dpSet(configName+".queryTimeout",queryTimeout);
        }
      }
    }
  }
  
  fwDevice_setAlert ( fwExternalApps_BASE_NODE + fwDevice_HIERARCHY_SEPARATOR + systemType ,
                      fwDevice_ALERT_SET , exceptionInfo );
  
  if ( dynlen(exceptionInfo) > 0 ) { return; }
  
  fwExternalApps_getAllActiveFromTree ( XmlTable , dpsSubsProd , dpsToCreate , dpsToCrType , 
                                        dpsToCrDept , dpsToParent , dpeToCrLeaf , exceptionInfo );

  if ( dynlen(exceptionInfo) > 0 ) { return; }

  fwExternalApps_createNodesWithType ( systemType , nodeTypes , dpsToCreate , dpsToCrType , 
                                       dpsToCrDept , dpsToParent , dpeToCrLeaf , tracing , exceptionInfo );

  if ( dynlen(exceptionInfo) > 0 ) { return; }
  
  for ( idx = 1 ; idx <= dynlen(dpsSubsProd) ; ++idx )
  {
    elems = strsplit ( dpsSubsProd[idx] , "|" );
    
    cnt = (int)elems[2];
    ent = (int)elems[3];
    
    if ( dynlen(elems) > 3 )
    {
      fwException_raise(exceptionInfo, "WARNING", "Datapoint '"+dpsSubsProd[idx]+"' contains '"+(string)(dynlen(elems)-3)+"' vertical bars!", "");
      continue;
    }
    
    thedata = tbl[cnt][fwExternalApps_NODES_DATA_INDEX];
     
    // The following type can only be fwExternalApps_DATA_TYPE_SUBSCR or fwExternalApps_DATA_TYPE_PUBLIC
    subpub_sort = thedata[ent][fwExternalApps_DATA_SUBS_PUBL];
    subpub_item = thedata[ent][fwExternalApps_DATA_SUBPUB_NAME];
    subpub_type = thedata[ent][fwExternalApps_DATA_SUBPUB_TYPE];
    subpub_buff = thedata[ent][fwExternalApps_DATA_BUFF_MODE];
    subpub_over = (thedata[ent][fwExternalApps_DATA_OVER_WRITE]?1:0);
    dpelem_name = thedata[ent][fwExternalApps_DATA_PNT_NAME]
                + thedata[ent][fwExternalApps_DATA_PNT_ELEMENT];
    subpub_tagn = thedata[ent][fwExternalApps_DATA_PNT_TAGNAME];

    // Use the following offsets in the data to created the appropriate Datapoint Element
    // fwExternalApps_DATA_SUBS_PUBL         - subpub_sort
    // fwExternalApps_DATA_SUBPUB_NAME       - subpub_item
    // fwExternalApps_DATA_SUBPUB_TYPE       - subpub_type
    // fwExternalApps_DATA_BUFF_MODE         - subpub_buff
    // fwExternalApps_DATA_OVER_WRITE        - subpub_over
    // fwExternalApps_DATA_PNT_NAME          - dpelem_name
    // fwExternalApps_DATA_PNT_ALIAS
    // fwExternalApps_DATA_PNT_DESCRIP
    // fwExternalApps_DATA_PNT_ELEMENT       - dpelem_name
    // fwExternalApps_DATA_PNT_TAGNAME       - subpub_tagn
    // fwExternalApps_DATA_PNT_TYPNAME
    // fwExternalApps_DATA_PNT_UNITS
    // fwExternalApps_DATA_PNT_MINERROR
    // fwExternalApps_DATA_PNT_MINWARN
    // fwExternalApps_DATA_PNT_MAXWARN
    // fwExternalApps_DATA_PNT_MAXERROR
      
    // rubbish...  not used...     dp_name = elems[1];
    
    if ( ( subpub_type != "single" ) && ( subpub_type != "multiple" ) )
    {
      fwException_raise(exceptionInfo, "WARNING", "Subscr/Public '"+subpub_item+"' has invalid Type '"+subpub_type+"'", "");
      continue;
    }
      
    if ( ( subpub_sort != fwExternalApps_DATA_TYPE_SUBSCR ) && ( subpub_sort != fwExternalApps_DATA_TYPE_PUBLIC ) )
    {
      fwException_raise(exceptionInfo, "WARNING", "Subscr/Public '"+subpub_item+"' Must be a Subscr or Public! '", "");
      continue;
    }

    if ( subpub_sort == fwExternalApps_DATA_TYPE_SUBSCR )
    {
      if ( subpub_over == 0 )
      {
        dynAppend ( sub_name_all_0 , dpelem_name );
        dynAppend ( sub_item_all_0 , subpub_item );
        dynAppend ( sub_tags_all_0 , ( subpub_type == "single" ? "" : subpub_tagn ) );
      }
      else
      {
        dynAppend ( sub_name_all_1 , dpelem_name );
        dynAppend ( sub_item_all_1 , subpub_item );
        dynAppend ( sub_tags_all_1 , ( subpub_type == "single" ? "" : subpub_tagn ) );
      }

      if (  ( thedata[ent][fwExternalApps_DATA_PNT_MINERROR] != "" )
         || ( thedata[ent][fwExternalApps_DATA_PNT_MINWARN]  != "" )
         || ( thedata[ent][fwExternalApps_DATA_PNT_MAXWARN]  != "" )
         || ( thedata[ent][fwExternalApps_DATA_PNT_MAXERROR] != "" ) )
      {
      dyn_float alertLimits = makeDynFloat();
      dyn_string alertTexts = makeDynString();
      dyn_string alertClasses = makeDynString();

        if ( thedata[ent][fwExternalApps_DATA_PNT_MINERROR] != "" )
        {
          dynAppend ( alertLimits , thedata[ent][fwExternalApps_DATA_PNT_MINERROR] );
          dynAppend ( alertTexts , "Exceeded minimum error threshold" );
          dynAppend ( alertClasses , "_fwErrorAck." );
        }

        if ( thedata[ent][fwExternalApps_DATA_PNT_MINWARN] != "" )
        {
          dynAppend ( alertLimits , thedata[ent][fwExternalApps_DATA_PNT_MINWARN] );
          dynAppend ( alertTexts , "Exceeded minimum warning threshold" );
          dynAppend ( alertClasses , "_fwWarningAck." );
        }
        
        dynAppend(alertTexts, "OK");
        dynAppend(alertClasses, "");
          
        if ( thedata[ent][fwExternalApps_DATA_PNT_MAXWARN] != "" )
        {
          dynAppend ( alertLimits , thedata[ent][fwExternalApps_DATA_PNT_MAXWARN] );
          dynAppend ( alertTexts , "Exceeded maximum warning threshold" );
          dynAppend ( alertClasses , "_fwWarningAck." );
        }
        
        if ( thedata[ent][fwExternalApps_DATA_PNT_MAXERROR] != "" )
        {
          dynAppend ( alertLimits , thedata[ent][fwExternalApps_DATA_PNT_MAXERROR] );
          dynAppend ( alertTexts , "Exceeded maximum error threshold" );
          dynAppend ( alertClasses , "_fwErrorAck." );
        }
        
        fwAlertConfig_set ( dpelem_name , DPCONFIG_ALERT_NONBINARYSIGNAL ,
                            alertTexts , alertLimits , alertClasses ,
                            makeDynString() , "" , makeDynString() , "" ,
                            strException , TRUE , TRUE , "." );
      }
    }
    else
    {
      if ( subpub_type == "single" )
      {
        if ( subpub_over == 0 )
        {
          dynAppend ( pub_sgl_name_all_0 , dpelem_name );
          dynAppend ( pub_sgl_item_all_0 , subpub_item );
          dynAppend ( pub_sgl_buff_all_0 , subpub_buff );
        }
        else
        {
          dynAppend ( pub_sgl_name_all_1 , dpelem_name );
          dynAppend ( pub_sgl_item_all_1 , subpub_item );
          dynAppend ( pub_sgl_buff_all_1 , subpub_buff );
        }
      }
      else
      {
        for ( i = 1 ; i <= dynlen(pub_str_all) ; ++i )
        {
          if ( subpub_item == pub_str_all[i][1] ) break;
        }
          
        if ( i > dynlen(pub_str_all) )
        {
          if ( subpub_over == 0 ) ++pub_str_0; else ++pub_str_1;
          
          dynAppend ( pub_str_all , makeDynAnytype(subpub_item,subpub_buff,subpub_over,makeDynString(),makeDynString()) );
        }
        else
        {
          if ( subpub_buff != pub_str_all[i][2] )
          {
            fwException_raise(exceptionInfo, "WARNING", "Publication tag '"+subpub_item+"' '"+subpub_tagn
                          +"' has different buff-mode '"+subpub_buff+"' vs '"+pub_str_all[i][2]+"'", "");
            continue;
          }
          if ( subpub_over != pub_str_all[i][3] )
          {
            fwException_raise(exceptionInfo, "WARNING", "Publication tag '"+subpub_item+"' '"+subpub_tagn
                          +"' has different overwrite-mode '"+subpub_over+"' vs '"+pub_str_all[i][3]+"'", "");
            continue;
          }
        }
          
        if ( pub_str_all[i][3] == 0 ) ++pub_tag_0; else ++pub_tag_1;
              
        dynAppend(pub_str_all[i][4],dpelem_name);
        dynAppend(pub_str_all[i][5],subpub_tagn);
      }
    }
        
    if ( thedata[ent][fwExternalApps_DATA_PNT_DESCRIP] == "" )
    {
      // Construct the Description from the subscription and Tagname...
      thedata[ent][fwExternalApps_DATA_PNT_DESCRIP] = "Subs-Publ" + " " + subpub_item + " - " + "Tagname" + " " + subpub_tagn;
      strreplace ( thedata[ent][fwExternalApps_DATA_PNT_DESCRIP] , "/" , " " );
    }
    if ( ( subpub_sort != fwExternalApps_DATA_TYPE_PUBLIC ) || ( dpGetDescription ( dpelem_name , -2 ) == "" ) )
    {
      dpSetDescription ( dpelem_name , thedata[ent][fwExternalApps_DATA_PNT_DESCRIP] );
    }
    
    if ( thedata[ent][fwExternalApps_DATA_PNT_UNITS] != "" )
       { dpSetUnit ( dpelem_name , thedata[ent][fwExternalApps_DATA_PNT_UNITS] ); }
  }
 
  if ( dynlen(exceptionInfo) > 0 ) { return; }

  if ( tracing > 0 ) DebugTN("Number of Subscriptions [Overwrite=False] are '"+dynlen(sub_name_all_0)+"'");
  
  fwDIP_subscribeMany ( sub_name_all_0 , configName , sub_item_all_0 , sub_tags_all_0 , exceptionInfo , FALSE );

  if ( tracing > 0 ) DebugTN("Number of Subscriptions [Overwrite=True]  are '"+dynlen(sub_name_all_1)+"'");

  fwDIP_subscribeMany ( sub_name_all_1 , configName , sub_item_all_1 , sub_tags_all_1 , exceptionInfo , TRUE );
  
  if ( tracing > 0 ) DebugTN("Number of Primitive(single) Pubs [O/W=F]  are '"+dynlen(pub_sgl_name_all_0)+"'");
  for ( idx = 1 ; idx <= dynlen(pub_sgl_name_all_0) ; ++idx )
  {
    fwDIP_publishPrimitive ( pub_sgl_item_all_0[idx] , pub_sgl_name_all_0[idx] , pub_sgl_buff_all_0[idx] , configName , strException , FALSE );
  }
  
  if ( tracing > 0 ) DebugTN("Number of Primitive(single) Pubs [O/W=T]  are '"+dynlen(pub_sgl_name_all_1)+"'");
  for ( idx = 1 ; idx <= dynlen(pub_sgl_name_all_1) ; ++idx )
  {
    // DebugTN ( "Publication Primitive" , ub_sgl_item_all_1[idx] , pub_sgl_name_all_1[idx] , pub_sgl_buff_all_1[idx] , configName , strException , TRUE );
    
    fwDIP_publishPrimitive ( pub_sgl_item_all_1[idx] , pub_sgl_name_all_1[idx] , pub_sgl_buff_all_1[idx] , configName , strException , TRUE );
  }
  
  if ( tracing > 0 ) DebugTN("Nbr of Structured(multiple) Pubs [O/W=F]  are '"+pub_str_0+"' [tags="+pub_tag_0+"]");
  if ( tracing > 0 ) DebugTN("Nbr of Structured(multiple) Pubs [O/W=T]  are '"+pub_str_1+"' [tags="+pub_tag_1+"]");
  
  for ( idx = 1 ; idx <= dynlen(pub_str_all) ; ++idx )
  {
    // DebugTN ( "Publication Structure" , pub_str_all[idx][1] , pub_str_all[idx][4] , pub_str_all[idx][5] ,
    //                                     pub_str_all[idx][2] , configName , strException , pub_str_all[idx][3] );
           
    fwDIP_publishStructure ( pub_str_all[idx][1] , pub_str_all[idx][4] , pub_str_all[idx][5] ,
                             pub_str_all[idx][2] , configName , strException , pub_str_all[idx][3] );
  }

  /* -------
  // Dumping to a File...
  string fn = getPath(DATA_REL_PATH);
  fn += systemType + "_" + "sub_new.txt";
  file fd = fopen(fn,"w");
  string bb;
  int i;
  for ( i = 1 ; i <= dynlen(sub_name_all_0) ; ++i )
  {
    sprintf(bb,"0 - %s - %s - %s\n",sub_name_all_0[i],sub_item_all_0[i],sub_tags_all_0[i]);
    fputs(bb,fd);
  }
  for ( i = 1 ; i <= dynlen(sub_name_all_1) ; ++i )
  {
    sprintf(bb,"1 - %s - %s - %s\n",sub_name_all_1[i],sub_item_all_1[i],sub_tags_all_1[i]);
    fputs(bb,fd);
  }
  fclose(fd);  
  // ... until here!
  ------- */
}
    

int _fwExternalApps_insertDataElement ( dyn_dyn_anytype & tbl , int entry , dyn_string elems , 
                                        int sub_pub , dyn_anytype thedata , 
                                        dyn_string & exceptionInfo )
{
int eleml = dynlen(elems);
int child,lastc;
int error = 0;
string elemt;

  if ( eleml < 1 ) { return(-1); }

  // The top-root entry in the tree is '1'
  if ( entry < 1 ) { entry = 1; }
  
  if ( eleml == 1 )
  {
    // Deal with the leaf element
    
    elemt = elems[1];
    child = tbl[entry][fwExternalApps_FRST_CHILD_INDEX];

    if ( child == 0 )
    {
      // Adding leaf element as the only child
      child = _fwExternalApps_createNewTableEntry(tbl, elemt);
      tbl[entry][fwExternalApps_FRST_CHILD_INDEX] = child;
      tbl[entry][fwExternalApps_LAST_CHILD_INDEX] = child;
    
      tbl[child][fwExternalApps_PARENTNODE_INDEX] = entry;
      tbl[child][fwExternalApps_ELEM_ACTIV_INDEX] = 1;
      _fwExternalApps_mergeDataInTable(tbl, child, sub_pub, thedata);
      
      for ( int part = entry ; part != 0 ; part = tbl[part][fwExternalApps_PARENTNODE_INDEX] )
      {
        ++tbl[part][fwExternalApps_CHILDN_ACT_INDEX];
        ++tbl[part][fwExternalApps_NBCHILDREN_INDEX];
      }
    }
    else
    {
      // Search for the existance of the child
      while ( child != 0 )
      {
        if ( tbl[child][fwExternalApps_TREE_ELEMT_INDEX] == elemt ) { break; }
        lastc = child;
        child = tbl[child][fwExternalApps_NEXT_SIBLG_INDEX];
      }
      
      if ( child == 0 )
      {
        // Adding leaf element as a new sibling
        child = _fwExternalApps_createNewTableEntry(tbl, elemt);
        tbl[entry][fwExternalApps_LAST_CHILD_INDEX] = child;

        tbl[lastc][fwExternalApps_NEXT_SIBLG_INDEX] = child;
      
        tbl[child][fwExternalApps_PREV_SIBLG_INDEX] = lastc;
        tbl[child][fwExternalApps_PARENTNODE_INDEX] = entry;
        tbl[child][fwExternalApps_ELEM_ACTIV_INDEX] = 1;
        _fwExternalApps_mergeDataInTable(tbl, child, sub_pub, thedata);
        
        for ( int part = entry ; part != 0 ; part = tbl[part][fwExternalApps_PARENTNODE_INDEX] )
        {
          ++tbl[part][fwExternalApps_CHILDN_ACT_INDEX];
          ++tbl[part][fwExternalApps_NBCHILDREN_INDEX];
        }
      }
      else
      {
        _fwExternalApps_mergeDataInTable(tbl, child, sub_pub, thedata);
      }
    }    
  }
  else
  {
    // More than one elements in the path
    
    elemt = elems[1];
    child = tbl[entry][fwExternalApps_FRST_CHILD_INDEX];
    
    if ( child == 0 )
    {
      // Adding node element as the only child
      child = _fwExternalApps_createNewTableEntry(tbl, elemt);
      tbl[entry][fwExternalApps_FRST_CHILD_INDEX] = child;
      tbl[entry][fwExternalApps_LAST_CHILD_INDEX] = child;
    
      tbl[child][fwExternalApps_PARENTNODE_INDEX] = entry;
      tbl[child][fwExternalApps_ELEM_ACTIV_INDEX] = 1;
      
      for ( int part = entry ; part != 0 ; part = tbl[part][fwExternalApps_PARENTNODE_INDEX] )
      {
        ++tbl[part][fwExternalApps_CHILDN_ACT_INDEX];
        ++tbl[part][fwExternalApps_NBCHILDREN_INDEX];
      }
      
      dynRemove(elems,1);
      error = _fwExternalApps_insertDataElement(tbl, child, elems, sub_pub, thedata, exceptionInfo);
    }
    else
    {
      // Search for the existance of the child
      while ( child != 0 )
      {
        if ( tbl[child][fwExternalApps_TREE_ELEMT_INDEX] == elemt ) { break; }
        lastc = child;
        child = tbl[child][fwExternalApps_NEXT_SIBLG_INDEX];
      }
      
      if ( child == 0 )
      {
        // Adding node element as a new sibling
  
        child = _fwExternalApps_createNewTableEntry(tbl, elemt);
        tbl[entry][fwExternalApps_LAST_CHILD_INDEX] = child;

        tbl[lastc][fwExternalApps_NEXT_SIBLG_INDEX] = child;
      
        tbl[child][fwExternalApps_PREV_SIBLG_INDEX] = lastc;
        tbl[child][fwExternalApps_PARENTNODE_INDEX] = entry;
        tbl[child][fwExternalApps_ELEM_ACTIV_INDEX] = 1;
        
        for ( int part = entry ; part != 0 ; part = tbl[part][fwExternalApps_PARENTNODE_INDEX] )
        {
          ++tbl[part][fwExternalApps_CHILDN_ACT_INDEX];
          ++tbl[part][fwExternalApps_NBCHILDREN_INDEX];
        }
      }    

      dynRemove(elems,1);
      error = _fwExternalApps_insertDataElement(tbl, child, elems, sub_pub, thedata, exceptionInfo);
    }  
  }
  
  return ( error );
}


int fwExternalApps_insertElementInTable ( dyn_dyn_anytype & tbl, dyn_string elems , 
                                          int sub_pub , dyn_anytype thedata , 
                                          dyn_string & exceptionInfo )
{
int error;

  if ( dynlen ( tbl ) == 0 )
  {
    fwException_raise(exceptionInfo, "WARNING", "Table into which Data is Inserted is Not Initialised!", "");
    error = -1;
  }
  else if ( tbl[1][fwExternalApps_NODES_DATA_INDEX] == 0 )
  {
    fwException_raise(exceptionInfo, "WARNING", "Table Already Displayed in Tree - No Further Data Insertion Possible!", "");
    error = -1;
  }
  else
  {
    error = _fwExternalApps_insertDataElement(tbl, -1, elems, sub_pub, thedata, exceptionInfo);
  }
  
  return ( error );
}
  

void _fwExternalApps_mergeDataInTable(dyn_dyn_anytype & tbl, int entry, 
                                      int sub_pub, dyn_anytype thedata)
{
int index = 0;
dyn_dyn_anytype all_data;

  if ( sub_pub == fwExternalApps_DATA_TYPE_NODATA ) return;
  
  if ( tbl[entry][fwExternalApps_NODESUBPUB_INDEX] == fwExternalApps_DATA_TYPE_NODATA )
  {
    tbl[entry][fwExternalApps_NODESUBPUB_INDEX] = sub_pub;
    all_data = makeDynAnytype();
  }
  else
  {
    if ( sub_pub != tbl[entry][fwExternalApps_NODESUBPUB_INDEX] )
    {
      tbl[entry][fwExternalApps_NODESUBPUB_INDEX] = fwExternalApps_DATA_TYPE_SUBPUB;
    }
    
    all_data = tbl[entry][fwExternalApps_NODES_DATA_INDEX];
    index = dynlen ( all_data );
  }
  
  all_data[index+1] = thedata;
  
  tbl[entry][fwExternalApps_NODES_DATA_INDEX] = all_data;
}


void fwExternalApps_showNodeDescription ( string treename , dyn_dyn_anytype & inf , 
                                          dyn_dyn_anytype & tbl , string id , int column )
{
shape tree = getShape(treename);
dyn_string exceptionInfo;
int entry = (int)id;
dyn_dyn_string data;
dyn_string elems;
int length;

  if ( column != 0 )
    tree.clear();
  else if ( tbl[entry][fwExternalApps_NODESUBPUB_INDEX] == fwExternalApps_DATA_TYPE_NODATA )
    tree.clear();
  else
  {
    data = tbl[entry][fwExternalApps_NODES_DATA_INDEX];

    length = dynlen(data);
    
    fwExternalApps_initialiseTable ( inf , exceptionInfo );
    
    for ( int idx = 1 ; idx <= length ; ++idx )
    {
      string subpub = ( data[idx][fwExternalApps_DATA_SUBS_PUBL] == fwExternalApps_DATA_TYPE_SUBSCR ? "Subscription: " : 
                      ( data[idx][fwExternalApps_DATA_SUBS_PUBL] == fwExternalApps_DATA_TYPE_PUBLIC ? "Publication: " : "Unknown - Bug! " ) );
      
      elems = makeDynString(subpub + data[idx][fwExternalApps_DATA_SUBPUB_NAME],
                            data[idx][fwExternalApps_DATA_PNT_TAGNAME],
                            "DP-Element: " + data[idx][fwExternalApps_DATA_PNT_ELEMENT]);
      
      fwExternalApps_insertElementInTable ( inf , elems, fwExternalApps_DATA_TYPE_NODATA , "" , exceptionInfo );

      
      elems = makeDynString(subpub + data[idx][fwExternalApps_DATA_SUBPUB_NAME],
                            data[idx][fwExternalApps_DATA_PNT_TAGNAME],
                            data[idx][fwExternalApps_DATA_PNT_DESCRIP]);
            
      fwExternalApps_insertElementInTable ( inf , elems, fwExternalApps_DATA_TYPE_NODATA , "" , exceptionInfo );
    }
    
    fwExternalApps_populateTree ( treename , inf , exceptionInfo , 0 );
  }
} 


/*
  
  The following routines initialise and populate the Tree
  fwExternalApps_initialiseTree is only called once at initialisation
  fwExternalApps_populateTree is called every time the Tree needs to be populated from scratch
  _fwExternalApps_populateSubTree is recursively called from the fwExternalApps_populateTree global routine
  
*/


void fwExternalApps_initialiseTree ( string treename , int mainTree = 1 )
{
int full_tree_width;
shape tree = getShape ( treename );

  tree.clear();
  
  tree.hScrollBarMode("Auto");
  tree.vScrollBarMode("Auto");
   
  // tree.setColumnWidthMode();
  tree.setResizeEnabled(true);
  
  tree.selectionMode("Single");
    
  full_tree_width = tree.width();
    
  if ( mainTree )
  {
    if ( fwExternalApps_DISPLAY_EXPANDED )
    {
      tree.addColumn ( "Tree Nodes & Leafs" );
      tree.addColumn ( "   Active" );
      tree.addColumn ( "    S/P" );
      tree.addColumn ( "     All" );
      tree.addColumn ( "   Some" );
      tree.addColumn ( "   None" );
    
      tree.setColumnWidth ( fwExternalApps_COLUMN_MAIN , full_tree_width - 280 - 5 );
      tree.setColumnWidth ( fwExternalApps_COLUMN__ACT , 60 );
      tree.setColumnWidth ( fwExternalApps_COLUMN_SUBS , 55 );
      tree.setColumnWidth ( fwExternalApps_COLUMN__ALL , 55 );
      tree.setColumnWidth ( fwExternalApps_COLUMN_SOME , 55 );
      tree.setColumnWidth ( fwExternalApps_COLUMN_NONE , 55 );
    }
    else
    {
      tree.addColumn ( "Tree Nodes & Leafs" );
      tree.addColumn ( "   Active" );
      tree.addColumn ( "    S/P" );
    
      tree.setColumnWidth ( fwExternalApps_COLUMN_MAIN , full_tree_width - 115 - 5 );
      tree.setColumnWidth ( fwExternalApps_COLUMN__ACT , 60 );
      tree.setColumnWidth ( fwExternalApps_COLUMN_SUBS , 55 );
    }
  }
  else
  {
    tree.showHeader(FALSE);
    
    tree.addColumn ( "Node's Description [Subs/Publ-Name Tag-Name Element-and-Description]" );
    
    tree.setColumnWidth ( fwExternalApps_COLUMN_MAIN , full_tree_width - 5 );
  }
}    
    
    
int _fwExternalApps_populateSubTree ( int mainTree , shape tree , dyn_dyn_anytype & tbl , int entry , dyn_string & exceptionInfo )
{
string parentstr;

  while ( entry != 0 )
  {
    parentstr = (string)tbl[entry][fwExternalApps_PARENTNODE_INDEX];
    if ( parentstr == "1" ) { parentstr = ""; }
    
    // DebugTN("Append '"+entry+"' to Parent '"+parentstr+"'");
    
    tree.appendItem ( parentstr , (string)entry , tbl[entry][fwExternalApps_TREE_ELEMT_INDEX] );
    tree.setOpen( (string)entry , TRUE );
    
    if ( mainTree )
    {
      tree.setIcon( (string)entry , fwExternalApps_COLUMN__ACT , fwExternalApps_COLUMN_ASSERT_ICON );
      _fwExternalApps_assertTreeColumn ( tree , tbl , entry );
      
      int dattyp = tbl[entry][fwExternalApps_NODESUBPUB_INDEX];
      string datstr = ( dattyp == fwExternalApps_DATA_TYPE_NODATA ? "       -" :
                      ( dattyp == fwExternalApps_DATA_TYPE_SUBSCR ? "  Subscr" :
                      ( dattyp == fwExternalApps_DATA_TYPE_PUBLIC ? " Publicat" : 
                      ( dattyp == fwExternalApps_DATA_TYPE_SUBPUB ? " Sub/Pub" : "    Bug!" ) ) ) );
      tree.setText( (string)entry , fwExternalApps_COLUMN_SUBS , datstr );
    }
    
    _fwExternalApps_populateSubTree ( mainTree , tree , tbl , tbl[entry][fwExternalApps_FRST_CHILD_INDEX] , exceptionInfo );
    
    entry = tbl[entry][fwExternalApps_NEXT_SIBLG_INDEX];
  }
  
  return ( 0 );
}
                     
                                 
int fwExternalApps_populateTree ( string treename , dyn_dyn_anytype & tbl , dyn_string & exceptionInfo , int mainTree = 1 )
{
int error = 0;
int first_child;
shape tree = getShape ( treename );
  
  if ( fwExternalApps_CHECK_PERFORMANCE ) DebugTN("Starts fwExternalApps_populateTree");

  tree.clear();
  
  if ( dynlen ( tbl ) == 0 )
  {
    fwException_raise(exceptionInfo, "WARNING", "Table to be Displayed in Tree Widget is Not Initialised!", "");
    error = -1;
  }
  else if ( ( first_child = tbl[1][fwExternalApps_FRST_CHILD_INDEX] ) == 0 )
  {
    // DebugTN("### Table to be Displayed in Tree Widget is Empty! ###");

    fwException_raise(exceptionInfo, "WARNING", "Table to be Displayed in Tree Widget is Empty!", "");
    error = -1;
  }
  else
  {
    // This Flag is Cleared because the Tree Widget Gets Now Populated
    // This Flag is Set when the Table Gets Initialised for Data Insertion
    tbl[1][fwExternalApps_NODES_DATA_INDEX] = 0;
    
    error = _fwExternalApps_populateSubTree ( mainTree , tree , tbl , first_child , exceptionInfo );
  }
  
  if ( fwExternalApps_CHECK_PERFORMANCE ) DebugTN("Ending fwExternalApps_populateTree");
    
  return ( error );
}


/*
  
  The following routines are specific for interogating the individual Tree widget's Nodes conditions
  fwExternalApps_isNodeFromTreeActive to check if a particular Node in the Tree is active

*/


int fwExternalApps_isNodeFromTreeActive ( dyn_dyn_anytype & tbl , dyn_string elems , dyn_string & exceptionInfo )
{
int length;
string elemt;
int child;
int entry = 1;
bool active = -1;

  for ( length = dynlen(elems) ; length != 0 ; --length )
  {
    elemt = elems[1];
    child = tbl[entry][fwExternalApps_FRST_CHILD_INDEX];
    while ( child != 0 )
    {
      if ( tbl[child][fwExternalApps_TREE_ELEMT_INDEX] == elemt ) { break; }
      child = tbl[child][fwExternalApps_NEXT_SIBLG_INDEX];
    }
    if ( child == 0 ) break;
    dynRemove(elems,1);
    entry = child;
  }
  
  if ( child != 0 ) active = tbl[entry][fwExternalApps_ELEM_ACTIV_INDEX];
  else
  {
    fwException_raise(exceptionInfo, "WARNING", "Node '"+(string)elems+"' could Not be Found in the Tree!", "");
  }
  return ( active );
}
    

void fwExternalApps_getAllActiveFromTree ( dyn_dyn_anytype & tbl , dyn_string & dpesubsprod , 
                                           dyn_string & dpstocreate , dyn_string & dpstocrtype , 
                                           dyn_int & dpstocrdept , dyn_string & dpstoparent , dyn_string & dpstocrleaf , 
                                           dyn_string & exceptionInfo )
{
int idx, cnt, ent, i;
dyn_string dpsubnames;
string dpname, dpsubname, dptype, dppart, dpleaf;

  for ( idx = 1 ; idx <= dynlen(tbl) ; ++idx )
  {
    if ( tbl[idx][fwExternalApps_ELEM_ACTIV_INDEX] == 0 ) continue;
    
    if ( tbl[idx][fwExternalApps_NODESUBPUB_INDEX] == fwExternalApps_DATA_TYPE_NODATA ) continue;
    
    for ( cnt = 1 ; cnt <= dynlen(tbl[idx][fwExternalApps_NODES_DATA_INDEX]) ; ++cnt )
    {
      dpname = tbl[idx][fwExternalApps_NODES_DATA_INDEX][cnt][fwExternalApps_DATA_PNT_NAME];
      dynAppend ( dpesubsprod , dpname + "|" + (string)idx + "|" + (string)cnt );
      
      dpsubnames = strsplit ( dpname , fwDevice_HIERARCHY_SEPARATOR );
      dpsubname = "";
      for ( i = 1 ; i <= dynlen(dpsubnames) ; ++i )
      {
        dppart = dpsubname;
        if ( i > 1 ) dpsubname += fwDevice_HIERARCHY_SEPARATOR;
        dpsubname += dpsubnames[i];
        dptype = tbl[idx][fwExternalApps_NODES_DATA_INDEX][cnt][fwExternalApps_DATA_PNT_TYPNAME];
        if ( ( ent = dynContains(dpstocreate,dpsubname) ) <= 0 )
        {
          // if ( i == dynlen(dpsubnames) ) DebugTN("@@@ Datapoint '"+dpname+"' got assigned the DP-Type '"+dptype+"'");
          dynAppend ( dpstocreate , dpsubname );
          dynAppend ( dpstocrtype , ( i == dynlen(dpsubnames) ? dptype : "" ) );
          dynAppend ( dpstocrdept , i );
          dynAppend ( dpstoparent , dppart );
          dynAppend ( dpstocrleaf , dpsubnames[i] );
        }
        else
        {
          if ( i == dynlen(dpsubnames) ) 
          {
            if ( dpstocrtype[ent] == "" )
            {
              // DebugTN("@@@ Datapoint '"+dpname+"' did not yet got DP-Type '"+dptype+"'");
              dpstocrtype[ent] = dptype;
            }
            else if ( dptype != dpstocrtype[ent] )
            {
              fwException_raise(exceptionInfo, "WARNING", "Datapoint '"+dpsubname+"' DP-Type mismatch - '"+dptype+"' vs '"+dpstocrtype[ent]+"'", "");
            }
          }
        }
      }
    }
  }
}


void fwExternalApps_createNodesWithType ( string systemType , dyn_string nodeTypes , 
                                          dyn_string dpstocreate , dyn_string dpstocrtype , 
                                          dyn_int dpstocrdept , dyn_string dpstoparent , dyn_string dpstocrleaf , 
                                          int tracing , dyn_string & exceptionInfo )
{
int idx, ent, depths, maxTypeDepth;
string dpname, dptype, dppart, dpleaf;
dyn_string elems, device, parent; 
int dp_datapoints_all = 0;
int dp_alarm_conf_all = 0;

  //
  // The dynamic-array 'nodeTypes' works in the following way...
  // Calculate first the depth of the Datapoint-Name
  // For depth 1 and 2, the DPs must already exist and have to be of type FwNode!
  // The array 'nodeTypes' correspond to depths 3, 4, 5, 6 ..., {last-DPT}
  // If the DPT falls outside the array or is the empty string, then the last entry in the array is taken!
  // It is as if the last DPT in the array is replicated forever...
  // An empty array is the same as an array of one DPT which corresponds in this case to FwExternalAppsNode
  //

  /* -------
  // Dumping to a File...
  string fn = getPath(DATA_REL_PATH);
  fn += systemType + "_" + "dps_new.txt";
  file fd = fopen(fn,"w");
  // ... until here!
  ------- */
  
  maxTypeDepth = dynlen(nodeTypes);

  for ( idx = 1 ; idx <= dynlen(dpstocreate) ; ++idx )
  {
    dpname = dpstocreate[idx];
    dptype = dpstocrtype[idx];
    depths = dpstocrdept[idx];
  
    if ( dptype == "" )
    {
      if ( depths < 3 ) { dptype = "FwNode"; }
    
      if ( ( dptype == "" ) && ( maxTypeDepth != 0 ) )
      {
        // Check the DP-Type in the Table "nodeTypes"
        if ( depths - 2 > maxTypeDepth )
          { dptype = nodeTypes[maxTypeDepth]; }
        else
        {
          dptype = nodeTypes[depths-2];
        
          if ( dptype == "" ) { dptype = nodeTypes[maxTypeDepth]; }
        }
      }
    
      // Use the Default DP-Type if it is still not defined...
      if ( dptype == "" ) { dptype = "FwExternalAppsNode"; }
      
      dpstocrtype[idx] = dptype;
    }
  }

  // Order the array of datapoints...
  
  for ( idx = 1 ; idx <= dynlen(dpstocreate) ; ++idx )
  {
    dpstocreate[idx] += "#" + (string)idx;
  }
  
  dynSortAsc(dpstocreate);
  
  DebugTN("Creating Datapoints...");
        
  for ( idx = 1 ; idx <= dynlen(dpstocreate) ; ++idx )
  {
    elems = strsplit ( dpstocreate[idx] , "#" );
    
    if ( dynlen(elems) > 2 )
    {
      fwException_raise(exceptionInfo, "WARNING", "Datapoint '"+dpstocreate[idx]+"' contains '"+(string)(dynlen(elems)-2)+"' hashes!", "");
      continue;
    }
    
    ent = (int)elems[2];
    dpname = elems[1];
    dptype = dpstocrtype[ent];
    depths = dpstocrdept[ent];
    dppart = dpstoparent[ent];
    dpleaf = dpstocrleaf[ent];
    
    /* -------
    // Dumping to a File...
    string bb;
    sprintf(bb,"%s - %s - %d - %s - %s\n",dpname,dptype,depths,dppart,dpleaf);
    fputs(bb,fd);
    // ... until here!
    ------- */
       
    // Check if the Datapoint already exists, otherwise create it now!
    
    if ( dpExists(dpname) == FALSE )
    {
      // DebugTN("Creating '"+dpname+"' of type '"+dptype+"'");

      device[fwDevice_DP_NAME] = dpleaf;
      device[fwDevice_DP_TYPE] = dptype;
      parent[fwDevice_DP_NAME] = dppart;
      parent[fwDevice_DP_TYPE] = "";
      
      fwDevice_create ( device , parent , exceptionInfo );
      
      if ( dynlen(exceptionInfo) > 0 )
      {
        if ( dpExists(dpname) == TRUE ) { dpDelete(dpname); }
        continue;
      }
      
      ++dp_datapoints_all;
      
      fwDevice_setAlert ( dpname , fwDevice_ALERT_SET , exceptionInfo );
      
      if ( dynlen(exceptionInfo) > 0 )
      { 
        if ( dpExists(dpname) == TRUE ) { dpDelete(dpname); }
        continue; 
      }

      ++dp_alarm_conf_all;
    }
  }
  
  /* -------
  // Dumping to a File...
  fclose(fd);  
  // ... until here!
  ------- */
  
  if ( tracing > 0 ) DebugTN("Number of Overall PVSS Datapoints Created are '"+dp_datapoints_all+"'");
  if ( tracing > 0 ) DebugTN("Number of DP Alarm Configurations Created are '"+dp_alarm_conf_all+"'");
}
  
  
/* ******* Danny - Unused

int fwExternalApps_getNextActiveFromTree ( dyn_dyn_anytype & tbl , int entry , int & depth , dyn_string & elems ,
                                           int & sub_pub , dyn_dyn_anytype & thedata , dyn_string & exceptionInfo )
{
int next,part;
  
  // The top-root entry in the tree is '1'
  if ( entry < 1 ) { entry = 1; if ( dynlen ( tbl ) == 0 ) return ( 0 ); }
  
  do
  {
    if (  ( ( next = tbl[entry][fwExternalApps_FRST_CHILD_INDEX] ) == 0 )
       && ( ( next = tbl[entry][fwExternalApps_NEXT_SIBLG_INDEX] ) == 0 ) )
    {
      if ( entry == 1 ) return ( 0 );
    
      while ( next == 0 )
      {
        entry = tbl[entry][fwExternalApps_PARENTNODE_INDEX];
     
        if ( entry == 1 ) return ( 0 );
     
        next = tbl[entry][fwExternalApps_NEXT_SIBLG_INDEX];
      }
    }
  
    entry = next;
  }
  while ( tbl[entry][fwExternalApps_ELEM_ACTIV_INDEX] == 0 );
  
  part = entry;
  elems = makeDynString();
  for ( depth = 0 ; part != 1 ; ++depth )
  {
    dynInsertAt(elems,tbl[part][fwExternalApps_TREE_ELEMT_INDEX],1);
    part = tbl[part][fwExternalApps_PARENTNODE_INDEX];
  }
  
  // enabled = ( tbl[entry][fwExternalApps_ELEM_ACTIV_INDEX] == 0 ? FALSE : TRUE );
      
  sub_pub = tbl[entry][fwExternalApps_NODESUBPUB_INDEX];
  
  if ( sub_pub != fwExternalApps_DATA_TYPE_NODATA )
  {
    thedata = tbl[entry][fwExternalApps_NODES_DATA_INDEX];
  }
  
  return ( entry );
}

- Danny - Unused ******* */


/* ******* Danny - Unused

int fwExternalApps_getNextNodeFromTree ( dyn_dyn_anytype & tbl , int entry , int & depth , dyn_string & elems ,
                                          bool & enabled , int & sub_pub , dyn_dyn_anytype & thedata , dyn_string & exceptionInfo )
{
int next,part;
  
  // The top-root entry in the tree is '1'
  if ( entry < 1 ) { entry = 1; if ( dynlen ( tbl ) == 0 ) return ( 0 ); }
  
  if (  ( ( next = tbl[entry][fwExternalApps_FRST_CHILD_INDEX] ) == 0 )
     && ( ( next = tbl[entry][fwExternalApps_NEXT_SIBLG_INDEX] ) == 0 ) )
  {
    if ( entry == 1 ) return ( 0 );
    
    while ( next == 0 )
    {
      entry = tbl[entry][fwExternalApps_PARENTNODE_INDEX];
     
      if ( entry == 1 ) return ( 0 );
     
      next = tbl[entry][fwExternalApps_NEXT_SIBLG_INDEX];
    }
  }
  
  entry = next;
  
  part = entry;
  elems = makeDynString();
  for ( depth = 0 ; part != 1 ; ++depth )
  {
    dynInsertAt(elems,tbl[part][fwExternalApps_TREE_ELEMT_INDEX],1);
    part = tbl[part][fwExternalApps_PARENTNODE_INDEX];
  }
  
  enabled = ( tbl[entry][fwExternalApps_ELEM_ACTIV_INDEX] == 0 ? FALSE : TRUE );
      
  sub_pub = tbl[entry][fwExternalApps_NODESUBPUB_INDEX];
  
  if ( sub_pub != fwExternalApps_DATA_TYPE_NODATA )
  {
    thedata = tbl[entry][fwExternalApps_NODES_DATA_INDEX];
  }
  
  return ( entry );
}

- Danny - Unused ******* */


/*
  
  The following routines are specific for the handling of operations onto a Tree widget structure
  fwExternalApps_openTree & _fwExternalApps_openSubTree to Collaps and Expand the Tree widget's structure
  fwExternalApps_treeRowColumnClicked when one clicks a row-column combination in the Tree widget's structure
  _fwExternalApps_activateCurrentNode & _fwExternalApps_activateAllBelow when one activates a Node in the Tree
  _fwExternalApps_desactivateCurrentNode & _fwExternalApps_desactivateAllBelow when one desactivates a Node in the Tree
  _fwExternalApps_assertTreeColumn asserts a Tree Column in the Tree widget's structure
  
*/
  
  
void _fwExternalApps_openSubTree ( shape tree , dyn_dyn_anytype & tbl , bool openTree , int entry )
{
  for ( int child = tbl[entry][fwExternalApps_FRST_CHILD_INDEX] ; child != 0 ; child = tbl[child][fwExternalApps_NEXT_SIBLG_INDEX] )
  {
    tree.setOpen( (string)child , openTree );
  
    _fwExternalApps_openSubTree ( tree , tbl , openTree , child );
  }
}


void fwExternalApps_openTree ( string treename , dyn_dyn_anytype & tbl , bool openTree , int entry = 1 )
{
shape tree = getShape ( treename );

  // _fwExternalApps_printTable ( tbl );

  if ( dynlen ( tbl ) != 0 )
  {
    tree.setOpen( ( entry == 1 ? "" : (string)entry ) , openTree );
  
    _fwExternalApps_openSubTree ( tree , tbl , openTree , entry );
  }
}      


void fwExternalApps_treeRowColumnClicked ( string treename , dyn_dyn_anytype & tbl , string id , int column )
{
shape tree = getShape ( treename );
int entry = (int)id;

  switch ( column )
  {
    case fwExternalApps_COLUMN_MAIN:
    {
      // Clicked on the Tree-Node lable - Display information related to this Item
      int x0 = 0;
    }
    break;
    case fwExternalApps_COLUMN_SUBS:
    {
      // Clicked on the Subscription/Publication column - do nothing yet
      int x1 = 0;
    }
    break;
    case fwExternalApps_COLUMN__ACT:
    {
      // Clicked on the Node-Active column
      if ( tbl[entry][fwExternalApps_ELEM_ACTIV_INDEX] == 0 )
      {
        _fwExternalApps_activateCurrentNode ( tree , tbl , entry );
      }
      else
      {
        _fwExternalApps_desactivateCurrentNode ( tree , tbl , entry );
      }
    }
    break;
    case fwExternalApps_COLUMN__ALL:
    {
      // Clicked on the Include-All column
      // if (  ( tbl[entry][fwExternalApps_NBCHILDREN_INDEX] != 0 )
      //    && ( tbl[entry][fwExternalApps_CHILDN_ACT_INDEX] != tbl[entry][fwExternalApps_NBCHILDREN_INDEX] ) )
      // {
      //   _fwExternalApps_activateAllChildrenOfNode ( tree , tbl , entry );
      // }

      // Clicked on the Include-Some column - do nothing yet
      int x3 = 0;
    }
    break;
    case fwExternalApps_COLUMN_SOME:
    {
      // Clicked on the Include-Some column - do nothing yet
      int x4 = 0;
    }
    break;
    case fwExternalApps_COLUMN_NONE:
    {
      // Clicked on the Include-None column
      // if (  ( tbl[entry][fwExternalApps_NBCHILDREN_INDEX] != 0 )
      //    && ( tbl[entry][fwExternalApps_CHILDN_ACT_INDEX] != 0 ) )
      // {
      //   _fwExternalApps_desactivateAllChildrenOfNode ( tree , tbl , entry );
      // }
        
      // Clicked on the Include-Some column - do nothing yet
      int x5 = 0;
    }
    break;
    default:
    {
      DebugTN("### Bug in XmlTable widget - Column '"+column+"' can Never be Clicked!");
    }
    break;
  }
}


void _fwExternalApps_activateAllBelow ( shape tree , dyn_dyn_anytype & tbl , int entry )
{
  tbl[entry][fwExternalApps_ELEM_ACTIV_INDEX] = 1;
  
  tbl[entry][fwExternalApps_CHILDN_ACT_INDEX] = tbl[entry][fwExternalApps_NBCHILDREN_INDEX];
  
  tree.setIcon( (string)entry , fwExternalApps_COLUMN__ACT , fwExternalApps_COLUMN_ASSERT_ICON );
  _fwExternalApps_assertTreeColumn ( tree , tbl , entry );
  
  for ( int child = tbl[entry][fwExternalApps_FRST_CHILD_INDEX] ; child != 0 ; child = tbl[child][fwExternalApps_NEXT_SIBLG_INDEX] )
  {
    _fwExternalApps_activateAllBelow ( tree , tbl , child );
  }
}

  
// This routine is Only called if the Current Node is Not Already Activated

void _fwExternalApps_activateCurrentNode ( shape tree , dyn_dyn_anytype & tbl , int entry )
{
int children_active;

  children_active = 1 + tbl[entry][fwExternalApps_NBCHILDREN_INDEX] - tbl[entry][fwExternalApps_CHILDN_ACT_INDEX];
  
  _fwExternalApps_activateAllBelow ( tree , tbl , entry );
  
  entry = tbl[entry][fwExternalApps_PARENTNODE_INDEX];
    
  while ( entry != 0 )
  {
    tbl[entry][fwExternalApps_CHILDN_ACT_INDEX] += children_active;
    
    if ( tbl[entry][fwExternalApps_ELEM_ACTIV_INDEX] == 0 )
    {
      tbl[entry][fwExternalApps_ELEM_ACTIV_INDEX] = 1; ++children_active;
      if ( entry != 1 ) tree.setIcon( (string)entry , fwExternalApps_COLUMN__ACT , fwExternalApps_COLUMN_ASSERT_ICON );
    }
    
    _fwExternalApps_assertTreeColumn ( tree , tbl , entry );
    
    entry = tbl[entry][fwExternalApps_PARENTNODE_INDEX];
  }
  
  // _fwExternalApps_printTable ( tbl );
}    
      
      
void _fwExternalApps_desactivateAllBelow ( shape tree , dyn_dyn_anytype & tbl , int entry )
{
  tbl[entry][fwExternalApps_ELEM_ACTIV_INDEX] = 0;
  
  tbl[entry][fwExternalApps_CHILDN_ACT_INDEX] = 0;
  
  tree.setIcon( (string)entry , fwExternalApps_COLUMN__ACT , fwExternalApps_COLUMN_DELETE_ICON );
  _fwExternalApps_assertTreeColumn ( tree , tbl , entry );

  for ( int child = tbl[entry][fwExternalApps_FRST_CHILD_INDEX] ; child != 0 ; child = tbl[child][fwExternalApps_NEXT_SIBLG_INDEX] )
  {
    _fwExternalApps_desactivateAllBelow ( tree , tbl , child );
  }
}

  
// This routine is Only called if the Current Node is Already Activated

void _fwExternalApps_desactivateCurrentNode ( shape tree , dyn_dyn_anytype & tbl , int entry )
{
int children_desact;

  children_desact = 1 + tbl[entry][fwExternalApps_CHILDN_ACT_INDEX];

  _fwExternalApps_desactivateAllBelow ( tree , tbl , entry );
  
  entry = tbl[entry][fwExternalApps_PARENTNODE_INDEX];

  while ( entry != 0 )
  {
    tbl[entry][fwExternalApps_CHILDN_ACT_INDEX] -= children_desact;
    
    if (  ( tbl[entry][fwExternalApps_ELEM_ACTIV_INDEX] != 0 )
       && ( tbl[entry][fwExternalApps_CHILDN_ACT_INDEX] == 0 )
       && ( tbl[entry][fwExternalApps_NODESUBPUB_INDEX] == fwExternalApps_DATA_TYPE_NODATA ) )
    {
      tbl[entry][fwExternalApps_ELEM_ACTIV_INDEX] = 0; ++children_desact;
      if ( entry != 1 ) tree.setIcon( (string)entry , fwExternalApps_COLUMN__ACT , fwExternalApps_COLUMN_DELETE_ICON );
    }
    
    _fwExternalApps_assertTreeColumn ( tree , tbl , entry );
    
    entry = tbl[entry][fwExternalApps_PARENTNODE_INDEX];
  }
  
  // _fwExternalApps_printTable ( tbl );
}    
      

void _fwExternalApps_assertTreeColumn ( shape tree , dyn_dyn_anytype & tbl, int entry )
{
int column = -1;
int nbchildren = tbl[entry][fwExternalApps_NBCHILDREN_INDEX];
int childn_act = tbl[entry][fwExternalApps_CHILDN_ACT_INDEX];
    
  if ( entry == 1 ) { return; }
  
  if ( nbchildren == 0 )
  {
    if ( childn_act != 0 )
    {
      DebugTN("### Software Error: Tree Table Entry '"+entry+"' Should have Zero Number of Active Children!");
      tree.setText ( (string)entry , fwExternalApps_COLUMN__ACT , "  Bug!" );
      _fwExternalApps_printTable ( tbl );
    }
    else
    {
      tree.setText ( (string)entry , fwExternalApps_COLUMN__ACT , "      -" );
    }
    column = 0;
  }
  else if ( childn_act < 0 )
  {
    DebugTN("### Software Error: Tree Table Entry '"+entry+"' Contains Negative Number of Active Children!");
    tree.setText ( (string)entry , fwExternalApps_COLUMN__ACT , "  Bug!" );
    _fwExternalApps_printTable ( tbl );
    column = 0;
  }
  else if ( childn_act == 0 )
  {
    tree.setText ( (string)entry , fwExternalApps_COLUMN__ACT , "  None" );
    column = fwExternalApps_COLUMN_NONE;
  }
  else if ( childn_act == nbchildren )
  {
    tree.setText ( (string)entry , fwExternalApps_COLUMN__ACT , "    All" );
    column = fwExternalApps_COLUMN__ALL;
  }
  else if ( childn_act > nbchildren )
  {
    DebugTN("### Software Error: Tree Table Entry '"+entry+"' Contains a Too High Number of Active Children!");
    tree.setText ( (string)entry , fwExternalApps_COLUMN__ACT , "  Bug!" );
    _fwExternalApps_printTable ( tbl );
    column = 0;
  }
  else
  {
    tree.setText ( (string)entry , fwExternalApps_COLUMN__ACT , "  Some" );
    column = fwExternalApps_COLUMN_SOME;
  }
  
  if ( fwExternalApps_DISPLAY_EXPANDED )
  {
    tree.setText ( (string)entry , fwExternalApps_COLUMN__ALL , ( column == 0 ? "          " : 
                   ( column == fwExternalApps_COLUMN__ALL ? "       X" : "        -" ) ) );
    tree.setText ( (string)entry , fwExternalApps_COLUMN_SOME , ( column == 0 ? "          " : 
                   ( column == fwExternalApps_COLUMN_SOME ? "       X" : "        -" ) ) );
    tree.setText ( (string)entry , fwExternalApps_COLUMN_NONE , ( column == 0 ? "          " : 
                   ( column == fwExternalApps_COLUMN_NONE ? "       X" : "        -" ) ) );
  }
}


