/**@name LIBRARY: pcConfigGW_FGC.ctl

@author: Herve Milcent (AB-CO)

Creation Date: 07/05/2007

Modification History: 
  
  24/06/2014: Jean-Charles
  - ENS-10480 Remove all references to FGC_MASK, counter, ST_FAULTS, ST_WARNINGS, COMPILE_TIME, WFIP_LOAD_RCVD, WFIP_LOAD_SENT

  15/11/2011: Herve
  - IS-656  FGC  GW_FGC_getFrontEndManagerConfig function to get the type of driver to show in the ImportDatabase panel. 
  
  30/05/2011: Herve
    - IS-552: Remove all dependencies on graphical object from the Import device panel in the core import 
    functions, to be able to use the unicos front-end/device check/import functions from a PVSS00ctrl 
    
  07/03/2011: Herve
  - IS-499: GW_FGC: CMW config entry not set if it is already set during the import. 
  
  19/02/2009: Herve
    - include pcGW_FGC.ctl libs
    
	- 02/07/2007: Herve
		function moved to unConfigGenericFunctions.ctl
		
		
version 1.0

External Functions: 
	
Internal Functions: 
	
Purpose: 
This library contains the function used to import the GW_FGC Front-End
	 	
Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
  . pcGeneration.cat
  . UNICOS-JCOP framework
  . PVSS version: 3.0
  . operating system: NT and W2000, but tested only under W2000.
  . distributed system: yes.
*/

//@{

//------------------------------------------------------------------------------------------------------------------------

// pcConfigGW_FGC_checkDeleteCommand
/**
Purpose: check the coherency of the delete command

Parameters:
  dsConfig, dyn_string, input, config dyn_string
  dsDeleteDps, dyn_string, output, list of dp that will be deleted
  exceptionInfo, dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

pcConfigGW_FGC_checkDeleteCommand(dyn_string dsConfig, dyn_string &dsDeleteDps, dyn_string &exceptionInfo)
{
	string sFeName, sPLCSubApplication = "", sObjectType = "", sNumber = "", sDpPlcName, sPlcModPlc;
	dyn_string dsAlarmDps;
	
	if (dynlen(dsConfig) >= 2)				// At least, delete instruction and plc name
	{
// get PLC Name
		sFeName = dsConfig[2];
		if ((sFeName != "") && (dsConfig[1] == UN_DELETE_COMMAND))
		{
			if (dynlen(dsConfig) >= 3)
			{
// get Application
				sPLCSubApplication = dsConfig[3];
			}
			if (dynlen(dsConfig) >= 4)
			{
// get DeviceType
				sObjectType = dsConfig[4];
			}
			if (dynlen(dsConfig) >= 5)
			{
// get device number
				sNumber = dsConfig[5];
			}
			unGenericDpFunctions_getPlcDevices("", sFeName, sPLCSubApplication, sObjectType, sNumber, dsDeleteDps);
// if only delete key word then delete also modbus and systemalarm
			if ((sFeName != "") && (sPLCSubApplication == "") && (sObjectType == "") && (sNumber == ""))
			{
// sDpPlc is GW_FGC_plcName
				sDpPlcName = c_pcSystemIntegrity_GW_FGC+sFeName;
				if (dpExists(sDpPlcName))
				{
					dynAppend(dsDeleteDps, sDpPlcName);
					dsAlarmDps = dpNames(c_unSystemAlarm_dpPattern + "*" + substr(sDpPlcName, strpos(sDpPlcName, ":") + 1) + "*",c_unSystemAlarm_dpType);
					dynAppend(dsDeleteDps, dsAlarmDps);
				}
			}
		}
		else
		{
			fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkDeleteCommand:" + getCatStr("unGeneration","BADINPUTS"),"");
		}
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkDeleteCommand:" + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------
//pcConfigGW_FGC_checkFrontEnd
/**
Purpose: check the coherency of the GW_FGC front-end config

Parameters:
  currentObject: string, input, type of config
  currentLineSplit: dyn_string, input, config
  dsDeleteDps: dyn_string, input, dp to be deleted
  exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
        . PVSS version: 3.0
        . operating system: NT and W2000, but tested only under W2000.
        . distributed system: yes.
*/
pcConfigGW_FGC_checkFrontEnd(string currentObject, dyn_string currentLineSplit, dyn_string dsDeleteDps, dyn_string &exceptionInfo)
{
	switch(currentObject) {
		case UN_PLC_COMMAND:
			pcConfigGW_FGC_checkApplication(currentLineSplit, dsDeleteDps, exceptionInfo);
//DebugN("checkFE", currentLineSplit[UN_CONFIG_GW_FGC_DEVICE]);
			break;
		case UN_FESYSTEMALARM_COMMAND:
 			fwException_raise(exceptionInfo, "ERROR", getCatStr("unGeneration","UNKNOWNFUNCTION"), "");
			break;
		default:
 			fwException_raise(exceptionInfo, "ERROR", getCatStr("unGeneration","UNKNOWNFUNCTION"), "");
			break;
	}
}

//------------------------------------------------------------------------------------------------------------------------

// pcConfigGW_FGC_deleteCommand
/**
Purpose: delete function

Parameters:
  dsConfig, dyn_string, input, config dyn_string
  exceptionInfo, dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

pcConfigGW_FGC_deleteCommand(dyn_string dsConfig, dyn_string &exceptionInfo)
{
	string sPLCSubApplication = "", sObjectType = "", sNumber = "";
	dyn_string dsDpeList, exceptionInfoTemp;
	
	pcConfigGW_FGC_checkDeleteCommand(dsConfig, dsDpeList, exceptionInfoTemp);
	if (dynlen(exceptionInfoTemp) > 0)
	{
		dynAppend(exceptionInfo, exceptionInfoTemp);
		fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_deleteCommand:" + getCatStr("unGeneration","BADINPUTS"),"");
	}
	else
	{
		if (dynlen(dsConfig) >= 2)				// At least, delete instruction and plc name
		{
			if (dynlen(dsConfig) >= 3)
			{
				sPLCSubApplication = dsConfig[3];
			}
			if (dynlen(dsConfig) >= 4)
			{
				sObjectType = dsConfig[4];
			}
			if (dynlen(dsConfig) >= 5)
			{
				sNumber = dsConfig[5];
			}
				
			unGenericDpFunctions_deletePlcDevices(dsConfig[2], sPLCSubApplication, sObjectType, sNumber, "progressBar", exceptionInfo);
			if ((dsConfig[2] != "") && (sPLCSubApplication == "") && (sObjectType == "") && (sNumber == ""))
			{
				pcConfigGW_FGC_deleteFrontEnd(dsConfig[2], exceptionInfo);
			}
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------
//pcConfigGW_FGC_setFrontEnd
/**
Purpose: set GW_FGC front-end config

Parameters:
  currentObject: string, input, type of config
  currentLineSplit: dyn_string, input, config
  exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcConfigGW_FGC_setFrontEnd(string currentObject, dyn_string currentLineSplit, dyn_string &exceptionInfo)
{
	string currentDevice, sPlcName, sPlcType, sAnalogArchive, sDpName;
	int iPlcNumber, driver;
	bool bEvent16;
	int iWorldFIP;
//DebugN("pcConfigGW_FGC_setFrontEnd", currentObject, currentLineSplit);

	switch(currentObject) {
		case UN_PLC_COMMAND:
			pcConfigGW_FGC_setApplication(currentLineSplit, exceptionInfo);
//DebugN("setFE", currentLineSplit[UN_CONFIG_GW_FGC_DEVICE]);
			break;
		case UN_FESYSTEMALARM_COMMAND:
 			fwException_raise(exceptionInfo, "ERROR", getCatStr("unGeneration","UNKNOWNFUNCTION"), "");
			break;
		default:
 			fwException_raise(exceptionInfo, "ERROR", getCatStr("unGeneration","UNKNOWNFUNCTION"), "");
			break;
	}
}

//------------------------------------------------------------------------------------------------------------------------

// pcConfigGW_FGC_checkApplication
/**
Purpose: check data for a sub application (not comm)

Parameters:
		dsPLC: dyn_string, input, GW_FGC configuration line in file + additional parameters
			dsPLC must contains :
			1. string, "QUANTUM" or "PREMIUM"
			2. string, PLC name
			3. string, PLC subapplication
			4. int, PLC address = PLC number
			(additional parameters)
			5. driver number 
			6. boolean archive name
			7. analog archive name
		dsDeleteDps, dyn_string, input, datapoints that will be deleted before importation
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

pcConfigGW_FGC_checkApplication(dyn_string dsPLC, dyn_string dsDeleteDps, dyn_string &exceptionInfo)
{
	dyn_string dsPLCLine, dsPLCAdditionalData;
	int i, iRes, iDriverNum;
	string sPLCDp, sHostname, sAliasDp, tempDp;
	
	if (dynlen(dsPLC) == (UN_CONFIG_GW_FGC_LENGTH + UN_CONFIG_PLC_ADDITIONAL_LENGTH))
		{
		dsPLCLine = dsPLC;
		for(i=1;i<=UN_CONFIG_PLC_ADDITIONAL_LENGTH;i++)
			{
			dynAppend(dsPLCAdditionalData, dsPLCLine[UN_CONFIG_GW_FGC_LENGTH+1]);
			dynRemove(dsPLCLine,UN_CONFIG_GW_FGC_LENGTH+1);
			}
		pcConfigGW_FGC_checkData(dsPLCLine, exceptionInfo);
		pcConfigGW_FGC_checkAdditionalData(dsPLCAdditionalData, exceptionInfo);

// first check if the PLC name is correct:
		tempDp = dsPLCLine[UN_CONFIG_PLC_NAME];
		if(!unConfigGenericFunctions_nameCheck(tempDp)) {
			fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkApplication: " + dsPLCLine[UN_CONFIG_PLC_NAME] + getCatStr("unGeneration","BADPLCNAME"),"");
			return;
		}

// check if the PLC Application Name is correct:
		tempDp = dsPLCLine[UN_CONFIG_PLC_SUBAPPLICATION];
		if(!unConfigGenericFunctions_nameCheck(tempDp)) {
			fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkApplication: " + dsPLCLine[UN_CONFIG_PLC_SUBAPPLICATION] + getCatStr("unGeneration","BADPLCAPPLICATIONNAME"),"");
			return;
		}
		
// sPLCDp is GW_FGC_plcName
		sPLCDp = getSystemName() +  c_pcSystemIntegrity_GW_FGC + dsPLCLine[UN_CONFIG_PLC_NAME];
//		sAliasDp = dpAliasToName(dsPLCLine[UN_CONFIG_PLC_NAME]);
		sAliasDp = unGenericDpFunctions_dpAliasToName(dsPLCLine[UN_CONFIG_PLC_NAME]);
//DebugN(sAliasDp, dsPLCLine[UN_CONFIG_PLC_NAME]);

		if (substr(sAliasDp, strlen(sAliasDp) - 1) == ".")
		{
			sAliasDp = substr(sAliasDp, 0, strlen(sAliasDp) - 1);
		}
		if ((sAliasDp != "") && (sAliasDp != sPLCDp))
		{
			if (dynContains(dsDeleteDps, sAliasDp) <= 0)		// Alias exists, is not used by the "GW_FGC" config datapoint and will not be deleted
			{
				fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkApplication: " + dsPLCLine[UN_CONFIG_PLC_NAME] + getCatStr("unGeneration","BADHOSTNAMEALIAS") + sAliasDp,"");
			}
		}
		if (dpExists(sPLCDp) && (dynContains(dsDeleteDps, sPLCDp) <= 0))
		{
			sHostname = unGenericDpFunctions_getAlias(sPLCDp);
			iRes = dpGet(sPLCDp + ".communication.driver_num", iDriverNum);
			if (iRes >= 0)
			{
				if (iDriverNum != dsPLCAdditionalData[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM])
				{
					fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkApplication: " + getCatStr("unGeneration","BADEXISTINGPLCDRIVER"),"");
				}
				if (sHostname != dsPLCLine[UN_CONFIG_PLC_NAME])
				{
					fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkApplication: " + getCatStr("unGeneration","BADEXISTINGPLCHOSTNAME"),"");
				}
			}
		}
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkApplication: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// pcConfigGW_FGC_checkData
/**
Purpose: check PLC data for configuration

Parameters:
		dsPLC: dyn_string, input, PLC configuration line in file
			dsPLC must contains :
			1. string, "QUANTUM" or "PREMIUM"
			2. string, name
			3. string, subApplication
			4. int, PLC address = PLC number
		exceptionInfo, dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

pcConfigGW_FGC_checkData(dyn_string dsPLC, dyn_string &exceptionInfo)
{
	if (dynlen(dsPLC) != UN_CONFIG_GW_FGC_LENGTH)
	{
		fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkData:" + getCatStr("unGeneration","BADINPUTS"),"");
	}
	else
	{
		unConfigGenericFunctions_checkPLCType(dsPLC[UN_CONFIG_PLC_TYPE], exceptionInfo);
		if (strrtrim(dsPLC[UN_CONFIG_PLC_NAME]) == "")
		{
			fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkData:" + getCatStr("unGeneration","ERRORPLCNAME"),"");
		}
		if (strrtrim(dsPLC[UN_CONFIG_PLC_SUBAPPLICATION]) == "")
		{
			fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkData:" + getCatStr("unGeneration","ERRORSUBAPPLI"),"");
		}			
	}
}

//------------------------------------------------------------------------------------------------------------------------

// pcConfigGW_FGC_checkAdditionalData
/**
Purpose: check additional data for configuration

Parameters:
		dsPLC: dyn_string, input, PLC additional data
			dsPLC must contains :
			1. unsigned, driver number
			2. string, boolean archive
			3. string, analog archive
		exceptionInfo, dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

pcConfigGW_FGC_checkAdditionalData(dyn_string dsPLC, dyn_string &exceptionInfo)
{
	unsigned uPLCDriver;
	
	if (dynlen(dsPLC) != UN_CONFIG_PLC_ADDITIONAL_LENGTH)
	{
		fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkAdditionalData:" + getCatStr("unGeneration","BADINPUTS"),"");
	}
	else
	{
		unConfigGenericFunctions_checkUnsigned(dsPLC[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM], uPLCDriver, exceptionInfo);
		if (dsPLC[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL] == "")
		{
			fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkAdditionalData: bool archive" + getCatStr("unGeneration","ERRORARCHIVEEMPTY"),"");
		}
		if (dsPLC[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_ANALOG] == "")
		{
			fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkAdditionalData: analog archive" + getCatStr("unGeneration","ERRORARCHIVEEMPTY"),"");
		}
		if (dsPLC[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_EVENT] == "")
		{
			fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_checkAdditionalData: event archive" + getCatStr("unGeneration","ERRORARCHIVEEMPTY"),"");
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------
// pcConfigGW_FGC_setApplication
/**
Purpose: set PLC configuration for a sub application (not comm)

Parameters:
		dsPLC: dyn_string, input, PLC configuration line in file + additional parameters
			dsPLC must contains :
			(Plc config line)
			1. string, "QUANTUM" or "PREMIUM"
			2. string, PLC name
			3. string, PLC subapplication
			4. int, PLC address = PLC number
			(additional parameters)
			5. driver number 
			6. boolean archive name
			7. analog archive name
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. First line of a generation file must contain the PLC description. Fields are defined in constants at this library beginning
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcConfigGW_FGC_setApplication(dyn_string dsPLCData, dyn_string &exceptionInfo)
{
	int iPLCNumber, iRes, i, position, iDriverNumber;
	string sDpName, addressReference, tempDp, sSection, sDriverLine;
	dyn_string dsPLC, dsAdditionalParameters, dsSubApplications, dsImportTimes, dsBoolArchives, dsAnalogArchives, exceptionInfoTemp;
	dyn_string dsEntry, dsEventArchives;
        dyn_string dsCMWSection;
	
	// Check
	
	if (dynlen(dsPLCData) == (UN_CONFIG_GW_FGC_LENGTH + UN_CONFIG_PLC_ADDITIONAL_LENGTH))
	{
	// 1. Separate config file info and additional parameters
		dsPLC = dsPLCData;
		dynClear(dsAdditionalParameters);
		for(i=1;i<=UN_CONFIG_PLC_ADDITIONAL_LENGTH;i++)
		{
			dynAppend(dsAdditionalParameters, dsPLC[UN_CONFIG_GW_FGC_LENGTH+1]);
			dynRemove(dsPLC,UN_CONFIG_GW_FGC_LENGTH+1);
		}
	
	// check if the PLC Application Name is correct:
		tempDp = dsPLCData[UN_CONFIG_PLC_SUBAPPLICATION];
		if(!unConfigGenericFunctions_nameCheck(tempDp)) {
			fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_setApplication: " + dsPLCData[UN_CONFIG_PLC_SUBAPPLICATION] + getCatStr("unGeneration","BADPLCAPPLICATIONNAME"),"");
			return;
		}
	
	// 3. Set PLC configuration in GW_FGC
		sDpName = c_pcSystemIntegrity_GW_FGC + dsPLC[UN_CONFIG_PLC_NAME];
		tempDp = sDpName;
		unConfigGenericFunctions_createDp(sDpName, PC_GW_FGC_DPTYPE, exceptionInfoTemp);
		if (dynlen(exceptionInfoTemp) <= 0)
		{
			sDpName = getSystemName() + sDpName;
			unConfigGenericFunctions_setAlias(sDpName, dsPLC[UN_CONFIG_PLC_NAME], exceptionInfo);
			iRes = dpGet(sDpName + ".configuration.subApplications", dsSubApplications,
						 sDpName + ".configuration.importTime", dsImportTimes,
						 sDpName + ".configuration.archive_bool", dsBoolArchives,
						 sDpName + ".configuration.archive_analog", dsAnalogArchives,
						 sDpName + ".configuration.archive_event", dsEventArchives);
			position = dynContains(dsSubApplications, dsPLC[UN_CONFIG_PLC_SUBAPPLICATION]);
			if (position <= 0)
			{
				position = dynlen(dsSubApplications) + 1;
			}
			dsSubApplications[position] = dsPLC[UN_CONFIG_PLC_SUBAPPLICATION];
			dsImportTimes[position] = getCurrentTime();
			dsBoolArchives[position] = dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL];
			dsAnalogArchives[position] = dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_ANALOG];
			dsEventArchives[position] = dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_EVENT];
	    iRes = dpSetWait(sDpName + ".configuration.importTime", dsImportTimes,
							 sDpName + ".configuration.archive_bool", dsBoolArchives,
							 sDpName + ".configuration.archive_analog", dsAnalogArchives,
							 sDpName + ".configuration.archive_event", dsEventArchives,
         	  	 sDpName + ".configuration.subApplications",dsSubApplications,
							 sDpName + ".communication.driver_num", dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM]);
			if (iRes < 0)
			{
				fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_setApplication: " + getCatStr("unPVSS","DPSETFAILED"),"");
			}	
		// set the WFIP BUS
			dpSetWait(sDpName+".configuration.WorldFIP", dsPLC[UN_CONFIG_GW_FGC_WFBUS]);

		// 4. Create _UnSystemAlarm datapoints
			unSystemIntegrity_createSystemAlarm(tempDp,
																					DS_pattern,
																					getCatStr("unSystemIntegrity", "PLC_DS_DESCRIPTION") + dsPLC[UN_CONFIG_PLC_NAME] + " -> DS driver " + dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM],
																					exceptionInfo);
			fwArchive_set(c_unSystemAlarm_dpPattern+DS_pattern+tempDp+".alarm",
						  dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL],
						  DPATTR_ARCH_PROC_SIMPLESM,
						  DPATTR_COMPARE_OLD_NEW,
						  0,
						  0,
						  exceptionInfo);
	
			unSystemIntegrity_createSystemAlarm(tempDp,
																					DS_Time_pattern,
																					getCatStr("unSystemIntegrity", "PLC_DS_TIME_DESCRIPTION") + dsPLC[UN_CONFIG_PLC_NAME] + " -> DS driver " + dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM],
																					exceptionInfo);
			fwArchive_set(c_unSystemAlarm_dpPattern+DS_Time_pattern+tempDp+".alarm",
						  dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL],
						  DPATTR_ARCH_PROC_SIMPLESM,
						  DPATTR_COMPARE_OLD_NEW,
						  0,
						  0,
						  exceptionInfo);

		// 5. CMW client config
			iDriverNumber = dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM];
                        iRes=fwInstallation_getSection( "CMWClient_"+iDriverNumber, dsCMWSection);
                        if(dynlen(dsCMWSection) <= 0)
			  _unCMWClient_setDriverConfig(iDriverNumber, "500", exceptionInfo);
		}
		else
		{
			dynAppend(exceptionInfo, exceptionInfoTemp);
		}
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","pcConfigGW_FGC_setApplication: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcConfigGW_FGC_deleteFrontEnd
/**
Purpose: delete Front-end config

Parameters : 
	sPlcName, string, input, plc name
	exceptionInfo, dyn_string, output, for errors
	
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

pcConfigGW_FGC_deleteFrontEnd(string sPlcName, dyn_string& exceptionInfo)
{
	string sDpPlcName, sPlcModPlc="", exceptionText;
	int i, length, iRes;
	dyn_string dsAlarmDps;
	

	sDpPlcName = c_pcSystemIntegrity_GW_FGC+sPlcName;
	if (!dpExists(sDpPlcName))
		sDpPlcName = "";

	if (sDpPlcName != "")
	{
		if (unGenericDpFunctions_getSystemName(sDpPlcName) == getSystemName())
		{
	// 2. Delete _Un_Plc
			iRes = dpDelete(substr(sDpPlcName, strpos(sDpPlcName, ":") + 1));
			if (iRes < 0)
			{
				fwException_raise(exceptionInfo, "ERROR", "pcConfigGW_FGC_deleteFrontEnd: " + substr(sDpPlcName, strpos(sDpPlcName, ":") + 1) + getCatStr("unGeneral", "UNOBJECTNOTDELETED"), "");
			}
	// 3. Delete alarm system 
			dsAlarmDps = dpNames(c_unSystemAlarm_dpPattern + "*" + substr(sDpPlcName, strpos(sDpPlcName, ":") + 1) + "*",c_unSystemAlarm_dpType);
			length = dynlen(dsAlarmDps);
			for(i=1;i<=length;i++)
			{
				_unSystemIntegrity_modifyApplicationAlertList(dsAlarmDps[i], false, exceptionInfo);
				iRes = dpDelete(dsAlarmDps[i]);
				if (iRes < 0)
				{
					fwException_raise(exceptionInfo, "ERROR", "pcConfigGW_FGC_deleteFrontEnd: " + dsAlarmDps[i] + getCatStr("unGeneral", "UNOBJECTNOTDELETED"), "");
				}
			}
			
		}
		else
		{
			fwException_raise(exceptionInfo, "ERROR", "pcConfigGW_FGC_deleteFrontEnd: " + sDpPlcName + getCatStr("unGeneral", "UNOBJECTNOTDELETEDBADSYSTEM"), "");
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// GW_FGC_checkFESystemAlarm
/**
Purpose: check FE system alarm config line for GW_FGC front-end

Parameters:
		dsConfig, dyn_string, input, config dyn_string
		exceptionInfo, dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

GW_FGC_checkFESystemAlarm(dyn_string dsConfig, dyn_string &exceptionInfo)
{
	fwException_raise(exceptionInfo,"ERROR","GW_FGC_checkFESystemAlarm:" + getCatStr("unGeneration","BADINPUTS"),"");
}

//------------------------------------------------------------------------------------------------------------------------

// GW_FGC_setFESystemAlarm
/**
Purpose: set FEsystem alarm config line form GW_FGC front-end

Parameters:
		dsConfig, dyn_string, input, config dyn_string
		dsAdditionalParameters, dyn_string, input, additional parameters
			1. dp name
			2. plc name
			3. plc type
			4. plc number
			5. driver num
		dsAddress, dyn_string, output, for address

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

GW_FGC_setFESystemAlarm(dyn_string dsConfig, dyn_string dsAdditionalParameters, dyn_string &dsAddress)
{
	fwException_raise(exceptionInfo,"ERROR","GW_FGC_setFESystemAlarm:" + getCatStr("unGeneration","BADINPUTS"),"");
}

//------------------------------------------------------------------------------------------------------------------------

// GW_FGC_getFrontEndArchiveDp
/**
Purpose: return the list of Dps that will be archived during the set of the GW_FGC front-end

Parameters:
		sCommand: string, input, the current entry in the import file
		sDp: string, input, the given dp: PLC or SystemAlarm  Dp
		sFEType: string, input, the front-end type
		dsArchiveDp: dyn_string, output, list of archivedDp

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

GW_FGC_getFrontEndArchiveDp(string sCommand, string sDp, string sFEType, dyn_string &dsArchiveDp)
{
//DebugN(sCommand, sDp, sFEType);
	switch(sCommand) {
		case UN_PLC_COMMAND:
			dsArchiveDp = makeDynString(getSystemName()+c_unSystemAlarm_dpPattern+DS_pattern+sDp+".alarm",
																		getSystemName()+c_unSystemAlarm_dpPattern+DS_Time_pattern+sDp+".alarm",
																		sDp);
			break;
		default:
			dsArchiveDp = makeDynString();
			break;
	}
}

//------------------------------------------------------------------------------------------------------------------------

//GW_FGC_getFrontEndManagerConfig
/**
Purpose: return the driver config of GW_FGC front-end

Parameters:
		iDriverNumber: 		int, 		input, 	the driver number
		sName: 					string, 		output,	the name to display
		sDriverName: 			string, 		output, 	the manager name
		iManType: 	int, output, the manager type
		sCommandLine: 	int, output, the command line to start the manager

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Win XP.
	. distributed system: yes.
*/

GW_FGC_getFrontEndManagerConfig(int iDriverNumber, string &sName, string &sDriverName, int &iManType, string &sCommandLine)
{
  string sDriverNumber;
  
  sprintf(sDriverNumber, "%02d", iDriverNumber);
  sName = "CMW";
  sDriverName = "PVSS00CMWClient";
  iManType = DRIVER_MAN;
  sCommandLine = "-num "+iDriverNumber+" -db CMWClient_Driver_"+sDriverNumber;
}

//------------------------------------------------------------------------------------------------------------------------
#uses "pcGW_FGC.ctl"

//@}
