/**@name LIBRARY: pcConfigGenericFunctions.ctl

@author:Herve Milcent(AB/CO)

Creation Date: 07-05-2007

Modification History:
	02/07/2007: Herve
		- remove functions
			pcConfigGenericFunctions_setDescription, pcConfigGenericFunctions_setPvRange
			pcConfigGenericFunctions_setUnit, pcConfigGenericFunctions_setFormat, pcConfigGenericFunctions_setArchiveConfig
			pcConfigGenericFunctions_setSmoothing
		
version 1.0

Purpose: 
This library contains all the definition of the PC devices.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 3.0
	. operating system: Linux and WXP but tested only on WXP.
	. distributed system: yes.
*/
//@{

//------------------------------------------------------------------------------------------------------------------------

// pcConfigGenericFunctions_setAddressConfig
/**
Purpose: set address config

Parameters:

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcConfigGenericFunctions_setAddressConfig(string sDriver, string sDpName, dyn_string dsDpe, string sCatalog, string sCMWDeviceName, string sCMWProperty, dyn_bool dbLLC, dyn_string &exceptionInfo)
{
	int i, len=dynlen(dsDpe);
	string sResult, sTag;
	dyn_string dsSplit;
	dyn_errClass err;
	
	if((len != dynlen(dbLLC)) || (len!=dynlen(dsDpe))) {
		fwException_raise(exceptionInfo,"ERROR","pcConfigGenericFunctions_setAddressConfig:" + getCatStr("unGeneration","BADINPUTS"),"");
		return;
	}
	for(i=1;i<=len;i++) {
		sResult = getCatStr(sCatalog, dsDpe[i]);
		err = getLastError();
		if(dynlen(err) <=0) {
			dsSplit=strsplit(sResult, ";");
			if(dynlen(dsSplit) == 3) {
				sTag = PC_CONFIG_CMW_TAG;
				if(dsSplit[3] != "")
					sTag +=":" + dsSplit[3];
				unConfigCMW_setAddress(sDpName + dsDpe[i], 
																sDriver, 
																dsSplit[1], 
																dsSplit[2],
																"",
																"",
																"",
																sCMWDeviceName,
																sCMWProperty,
																sTag,
																dbLLC[i],
																exceptionInfo);
			}
			else
				fwException_raise(exceptionInfo,"ERROR","pcConfigGenericFunctions_setAddressConfig: " + dsDpe[i]+ " " + getCatStr("unGeneration","BADINPUTS") + " " + sResult,"");
		}
		else {
			throwError(err);
			fwException_raise(exceptionInfo,"ERROR","pcConfigGenericFunctions_setAddressConfig: " + dsDpe[i]+ " " + getCatStr("unGeneration","BADINPUTS"),"");
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

//@}
